<?php

/**
 * InterWorx Hosting Control Panel
 *
 * <pre>
 * +----------------------------------------------------------------------+
 * | Copyright (c) 2000-2010 InterWorx L.L.C., All Rights Reserved.       |
 * +----------------------------------------------------------------------+
 * | Redistribution and use in source form, with or without modification  |
 * | is NOT permitted without consent from the copyright holder.          |
 * |                                                                      |
 * | THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS "AS IS" AND |
 * | ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,    |
 * | THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A          |
 * | PARTICULAR PURPOSE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,    |
 * | EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,  |
 * | PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR   |
 * | PROFITS; OF BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY  |
 * | OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT         |
 * | (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE    |
 * | USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH     |
 * | DAMAGE.                                                              |
 * +----------------------------------------------------------------------+
 * </pre>
 *
 * @package InterWorx
 * @author  Malin Lorch <mlorch@liquidweb.com>
 * @date    Jun 12, 2020
 */

/**
 *
 * @package Interworx
 * @subpackage Plugin_Acronis
 */
class Ctrl_Siteworx_Acronis extends Ctrl_Siteworx_Plugin
{
    const IS_AUTHORIZED = 'authorized';
    const CONTACT_ADMIN = 'display_contact_admin';
    const AB_MESSAGE_API_FAILURE = '##LG_ACRONIS_API_FAILURE##';
    const AB_MESSAGE_NO_BACKUP = '##LG_ACRONIS_NO_BACKUP##';
    const AB_MESSAGE_INVALID_PATH = '##LG_ACRONIS_INVALID_PATH##';
    const AB_MESSAGE_UNKNOWN_ERROR = '##LG_UNKNOWN_ERROR##';
    const AB_MESSAGE_NOT_AUTHORIZED = '##LG_ACRONIS_NOT_AUTHORIZED##';
    const ACTION_NOT_ALLOWED = '##LG_ACTION_BLOCKED##';
    const RESTORE_COMPLETE_ALT_LOCATION = '##LG_RESTORE_ALT_LOCATION_COMPLETE##';
    const DIRECTORIES_RECOVERED = '##LG_RESTORE_DIRECTORIES_COMPLETE##';
    const AB_MESSAGE_NO_DB = '##LG_ACRONIS_NO_DB##';

    private $_ActivitesLogger;

    /**
     * Init function. Final construct will run this method.
     */
    protected function _init()
    {
        $this->_Plugin = $this->_getPlugin();
    }

    /**
     * Allow actions if master user.
     *
     * @param string $action
     */
    public function isActionAllowed($action)
    {
        if (IW::SW()->getWorkingUser()->isMaster()) {
            return true;
        }

        return false;
    }

    /**
     * Load backups.
     */
    public function indexAction()
    {
        $isAuthorized = $this->_Plugin->userIsAuthorized();

        if (!$isAuthorized) {
            $this->_displayFailure('_notAuthorized', self::AB_MESSAGE_NOT_AUTHORIZED);
        }

        $BackupAgent = $this->_Plugin->getBackupAgent();
        $backupAgentNotInstalled = !$BackupAgent->isInstalled();
        $scriptsMissing = $BackupAgent->scriptsMissing();
        $isProtected = $this->_Plugin->getServer()->isProtected();

        if ($backupAgentNotInstalled || $scriptsMissing || !$isProtected) {
            $View = $this->getView();
            $View->assign(self::CONTACT_ADMIN, true);
        }

        // js handles what to display by xhr requests if installed & authorized
    }

    /**
     * Exists because index action can't refer to itself
     * when displayFailure is triggered.
     */
    private function _notAuthorizedAction()
    {
        $View = $this->getView();
        $View->assign(self::IS_AUTHORIZED, false);
    }

    /**
     * Get backups.
     *
     * @param Form_SubmitOnly $Form
     */
    public function listBackupsCommitAction(Form_SubmitOnly $Form)
    {
        $this->_executeFunctionAndCatchException('_listBackups', $Form);
    }

    /**
     * Get backups.
     *
     * @param Form_SubmitOnly $Form
     */
    private function _listBackups(Form_SubmitOnly $Form)
    {
        $BackupsRetriever = $this->_Plugin->getBackupsRetriever();
        $backups = $BackupsRetriever->getBackups();
        $this->_displayPayloadAsJson($backups);
    }

    /**
     * Get root content.
     *
     * @param Form_AcronisBackups $Form
     */
    public function getRootContentCommitAction(Form_AcronisBackups $Form)
    {
        $this->_executeFunctionAndCatchException('_getRootContent', $Form);
    }

    /**
     * Get root content.
     *
     * @param Form_AcronisBackups $Form
     */
    private function _getRootContent(Form_AcronisBackups $Form)
    {
        try {
            $backupId = $Form->getBackupId();
            $FileRecoveryPoint = $this->_Plugin->getSwFilesRecoveryPoint($backupId);
            $dirContent = $FileRecoveryPoint->getRootContent();
            $this->_displayPayloadAsJson($dirContent);
        } catch (Exception_InvalidPath $e) {
            $this->_logError(__FUNCTION__, $e);
            $this->_displayFailure('index', self::AB_MESSAGE_NO_BACKUP);
        }
    }

    /**
     * Get directory content by id.
     *
     * @param Form_AcronisBackups_ListDirectoryContent $Form
     */
    public function getDirectoryContentCommitAction(Form_AcronisBackups_ListDirectoryContent $Form)
    {
        $this->_executeFunctionAndCatchException('_getDirContent', $Form);
    }


    /**
     * Get directory content by id.
     *
     * @param Form_AcronisBackups_ListDirectoryContent $Form
     */
    private function _getDirContent(Form_AcronisBackups_ListDirectoryContent $Form)
    {
        $backupId = $Form->getBackupId();
        $dirId = $Form->getDirectoryPath();
        $FileRecoveryPoint = $this->_Plugin->getSwFilesRecoveryPoint($backupId);
        $dirContent = $FileRecoveryPoint->getDirectoryContent($dirId);
        $this->_displayPayloadAsJson($dirContent);
    }

    /**
     * Recover directories.
     *
     * @param Form_AcronisBackups_RecoverDirectories $Form
     */
    public function recoverDirectoriesCommitAction(Form_AcronisBackups_RecoverDirectories $Form)
    {
        $this->_executeFunctionAndCatchException('_recoverDirectories', $Form);
    }

    /**
     * Recover directories.
     *
     * @param Form_AcronisBackups_RecoverDirectories $Form
     */
    private function _recoverDirectories(Form_AcronisBackups_RecoverDirectories $Form)
    {
        $backupId = $Form->getBackupId();
        $dirPaths = $Form->getDirectoryPaths();
        $shouldDeleteOlderFiles = $Form->deleteRequested();
        $FileRecoveryPoint = $this->_Plugin->getSwFilesRecoveryPoint($backupId);
        $FileRecoveryPoint->recoverDirectories($dirPaths, $shouldDeleteOlderFiles);

        $this->_getActivitiesLogger()->logRestoreFiles($dirPaths);

        $Payload = new Payload(self::DIRECTORIES_RECOVERED);
        $this->_displayPayload($Payload);
    }

    /**
     * Get restore location.
     */
    public function getRestoreAltLocationAction()
    {
        $altLocation = Model_RecoveryPoint_Files_Siteworx::getRestoreAltLocationPath(IW::SW());
        $Payload = new Payload(array(
            'altLocation' => $altLocation
        ));

        $this->_displayPayload($Payload);
    }



    /**
     * Restore directories to alt location.
     *
     * @param Form_AcronisBackups_RestoreAltLocation $Form
     */
    public function restoreToAltLocationCommitAction(Form_AcronisBackups_RestoreAltLocation $Form)
    {
        $this->_executeFunctionAndCatchException('_restoreToAltLocation', $Form);
    }

    /**
     * Restore directories to alt location.
     *
     * @param Form_AcronisBackups_RestoreAltLocation $Form
     */
    private function _restoreToAltLocation(Form_AcronisBackups_RestoreAltLocation $Form)
    {
        $backupId = $Form->getBackupId();
        $dirPaths = $Form->getDirectoryPaths();
        $FileRecoveryPoint = $this->_Plugin->getSwFilesRecoveryPoint($backupId);
        $FileRecoveryPoint->restoreToAltLocation($dirPaths);

        $this->_getActivitiesLogger()->logDownloadFiles($dirPaths);

        $Payload = new Payload(self::RESTORE_COMPLETE_ALT_LOCATION);
        $this->_displayPayload($Payload);
    }

    /**
     * List databases.
     *
     * @param Form_AcronisBackups $Form
     */
    public function listDatabasesCommitAction(Form_AcronisBackups $Form)
    {
        $this->_executeFunctionAndCatchException('_listDatabases', $Form);
    }

    /**
     * List databases.
     *
     * @param Form_AcronisBackups $Form
     */
    private function _listDatabases(Form_AcronisBackups $Form)
    {
        $backupId = $Form->getBackupId();
        $unixUser = IW::SW()->getUniqname();
        $DbRestorer = $this->_Plugin->getSwDatabaseRecoveryPoint($backupId, $unixUser);
        $dbs = $DbRestorer->getDbs();

        $this->_displayPayloadAsJson($dbs);
    }

    /**
     * Restore datbases.
     * @param Form_AcronisBackups_Siteworx_RecoverDatabases $Form
     */
    public function restoreDatabasesCommitAction(Form_AcronisBackups_Siteworx_RecoverDatabases $Form)
    {
        $this->_executeFunctionAndCatchException('_restoreDatabases', $Form);
    }

    /**
     * Restore datbases.
     * @param Form_AcronisBackups_Siteworx_RecoverDatabases $Form
     */
    private function _restoreDatabases(Form_AcronisBackups_Siteworx_RecoverDatabases $Form)
    {
        $backupId = $Form->getBackupId();
        $databases = $Form->getDatabases();
        $unixUser = IW::SW()->getUniqname();
        $DbRestorer = $this->_Plugin->getSwDatabaseRecoveryPoint($backupId, $unixUser);
        $DbRestorer->restoreDatabases($databases);

        $this->_getActivitiesLogger()->logRestoreDatabases($databases);

        $Payload = new Payload('Databases successfully recovered');
        $this->_displayPayload($Payload);
    }

    /**
     * Restore datbases.
     * @param Form_AcronisBackups_Siteworx_RecoverDatabases $Form
     */
    public function restoreDatabasesAltLocationCommitAction(Form_AcronisBackups_Siteworx_RecoverDatabases $Form)
    {
        $this->_executeFunctionAndCatchException('_restoreDatabasesAltLocation', $Form);
    }

    /**
     * Restore datbases.
     * @param Form_AcronisBackups_Siteworx_RecoverDatabases $Form
     */
    private function _restoreDatabasesAltLocation(Form_AcronisBackups_Siteworx_RecoverDatabases $Form)
    {
        $backupId = $Form->getBackupId();
        $databases = $Form->getDatabases();
        $unixUser = IW::SW()->getUniqname();

        $DbRestorer = $this->_Plugin->getSwDatabaseRecoveryPoint($backupId, $unixUser);
        $DbRestorer->restoreDatabasesAltLocation($databases);

        $this->_getActivitiesLogger()->logDownloadDatabases($databases);

        $Payload = new Payload('Databases successfully recovered');
        $this->_displayPayload($Payload);
    }

    /**
     *
     * @param string $functionToCall
     * @param Form   $Form
     */
    private function _executeFunctionAndCatchException($functionToCall, Form $Form)
    {
        try {
            $this->$functionToCall($Form);
        } catch (Exception_DbNotFound $e) {
            $this->_logError($functionToCall, $e);
            $this->_displayFailure('index', self::AB_MESSAGE_NO_DB);
        } catch (Exception_AcronisApiFailure $e) {
            $this->_logError($functionToCall, $e);
            $this->_displayFailure('index', self::AB_MESSAGE_API_FAILURE);
        } catch (Exception_NoUnixUser $e) {
            $this->_logError($functionToCall, $e);
            $this->_displayFailure('index', self::AB_MESSAGE_NO_BACKUP);
        } catch (Exception_InvalidPath $e) {
            $this->_logError($functionToCall, $e);
            $this->_displayFailure('index', self::AB_MESSAGE_INVALID_PATH);
        } catch (Exception $e) {
            $this->_logError($functionToCall, $e);
            $this->_displayFailure('index', self::AB_MESSAGE_UNKNOWN_ERROR);
        }
    }

    /**
     * Display payload as json if CLI
     *
     * @param mixed $payloadContent
     */
    private function _displayPayloadAsJson($payloadContent)
    {
        // render as json
        if (IW::Env()->matches(IW::CLI)) {
            IW::Env()->setOutputStyle('pretty');
        }

        $Payload = new Payload($payloadContent);
        $this->_displayPayload($Payload);
    }

    /**
     * Log error msg.
     *
     * @param string    $functionName
     * @param Exception $e
     */
    private function _logError($functionName, Exception $e)
    {
        $exceptionDetails = $this->_getMessageFromException($e);
        IW::getLog()->error("Action failed $functionName: $exceptionDetails");
    }

    /**
     * Get message from exception
     *
     * @param Exception $e
     */
    private function _getMessageFromException(Exception $e)
    {
        $class = $e->getFile();
        $line = $e->getLine();
        $message = $e->getMessage();
        return "$message. File: $class, line: $line";
    }

    /**
     * get sw activities logger.
     *
     * @return Model_Logger_Activities_Siteworx
     */
    protected function _getActivitiesLogger()
    {
        if (!isset($this->_ActivitesLogger)) {
            $this->_ActivitesLogger = new Model_Logger_Activities_Siteworx();
        }

        return $this->_ActivitesLogger;
    }
}
