<?php
/**
 * InterWorx Hosting Control Panel
 *
 * <pre>
 * +----------------------------------------------------------------------+
 * | Copyright (c) 2000-2010 InterWorx L.L.C., All Rights Reserved.       |
 * +----------------------------------------------------------------------+
 * | Redistribution and use in source form, with or without modification  |
 * | is NOT permitted without consent from the copyright holder.          |
 * |                                                                      |
 * | THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS "AS IS" AND |
 * | ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,    |
 * | THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A          |
 * | PARTICULAR PURPOSE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,    |
 * | EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,  |
 * | PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR   |
 * | PROFITS; OF BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY  |
 * | OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT         |
 * | (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE    |
 * | USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH     |
 * | DAMAGE.                                                              |
 * +----------------------------------------------------------------------+
 * </pre>
 *
 * @package InterWorx
 * @author  Malin Lorch <mlorch@liquidweb.com>
 * @date    Mar 31, 2020
 */

/**
 * acronis Model.
 *
 * @package    InterWorx
 * @subpackage Plugin
 */
abstract class Model_Acronis
{
    private $_api;

    /**
     * Configure date.
     *
     * @param string $date
     * @return array
     */
    protected function _formatDate($date)
    {
        if (empty($date)) {
            return 0;
        }

        $time = strtotime($date);

        $weekday = date('l', $time);
        $day = date('d', $time);
        $month = date('F', $time);
        $year = date('Y', $time);
        $hour = date('H', $time);
        $minutes = date('i', $time);

        return array(
            'year' => $year,
            'month' => $month,
            'day' => $day,
            'weekday' => $weekday,
            'time' => "$hour:$minutes"
        );
    }


    /**
     * Converts bytes to GB and returns size with GB suffix.
     *
     * @param float $bytes
     * @return string
     */
    protected function _convertBytes($bytes)
    {
        if ($bytes === 0) {
            return '0 GB';
        }

        $units = array('B', 'KB', 'MB', 'GB', 'TB');

        $base = log($bytes) / log(1024);
        $suffix = $units[floor($base)];


        $result = pow(1024, $base - floor($base));

        $result = round($result, 1);
        return "$result $suffix";
    }

    /**
     * Get acronis plugin.
     *
     * @return Plugin
     */
    protected function _getPlugin()
    {
        return IW::PluginManager()->getPlugin('acronis');
    }

    /**
     * Get api.
     *
     * @param array $args
     * @return Model_Api
     */
    protected function _getApi(array $args = null)
    {
        if (!isset($this->_api)) {
            $ApiName = $this->_getApiClassName();
            $this->_api = $this->_getApiClass($ApiName, $args);
        }

        return $this->_api;
    }

    /**
     * Get api class.
     *
     * @param string $apiName
     * @param array  $args
     */
    private function _getApiClass($apiName, array $args = null)
    {
        if ($args) {
            return new $apiName(...$args);
        }


        return new $apiName();
    }

    /**
     * Get api class name.
     *
     * Expects api classname to be same
     * as class but with Api suffix,
     * e.g. Model_CloudStorage_Api
     *
     * @return string
     */
    private function _getApiClassName()
    {
        $className = static::class;

        $firstOccurance = strpos($className, '_') + 1;
        $secondOccurance = strpos(substr($className, $firstOccurance), '_') + $firstOccurance;

        $result = substr($className, 0, $secondOccurance) . '_Api';
        return $result;
    }

    /**
     * Get cache.
     */
    protected function _getCache()
    {
        return $this->_getPlugin()->getCache();
    }

    /**
     * check if dir exists.
     *
     * @param string $path
     * @return boolean
     */
    protected function _isDir($path)
    {
        $Command = new Command("[[ -d $path ]]");
        $Command->runAsRoot();
        $Command->allowBracketBoth();

        return $Command->exec();
    }

    /**
     * check if file exists.
     *
     * @param string $path
     * @return boolean
     */
    protected function _isFile($path)
    {
        $Command = new Command("[[ -f $path ]]");
        $Command->runAsRoot();
        $Command->allowBracketBoth();

        return $Command->exec();
    }
}
