<?php

/**
 * InterWorx Hosting Control Panel
 *
 * <pre>
 * +----------------------------------------------------------------------+
 * | Copyright (c) 2000-2010 InterWorx L.L.C., All Rights Reserved.       |
 * +----------------------------------------------------------------------+
 * | Redistribution and use in source form, with or without modification  |
 * | is NOT permitted without consent from the copyright holder.          |
 * |                                                                      |
 * | THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS "AS IS" AND |
 * | ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,    |
 * | THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A          |
 * | PARTICULAR PURPOSE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,    |
 * | EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,  |
 * | PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR   |
 * | PROFITS; OF BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY  |
 * | OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT         |
 * | (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE    |
 * | USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH     |
 * | DAMAGE.                                                              |
 * +----------------------------------------------------------------------+
 * </pre>
 *
 * @package InterWorx
 * @author  Malin Lorch <mlorch@liquidweb.com>
 * @date    Dec 05, 2019
 */

 require_once(__DIR__ . '/Interface.php');
/**
 * Acronis API client.
 *
 * @package Interworx
 * @subpackage Plugin_Acronis
 */
abstract class Model_ApiClient_Client implements Model_ApiClient_Interface
{
    const AB_COOKIEDIR = '/home/interworx/etc/env/ab_cookies';

    private $_baseUrl;
    private $_cURL;

    /**
     * Initiates curl.
     *
     * @param string $url
     */
    public function __construct($url)
    {
        $this->_baseUrl = $url;
    }


    /**
     * Initialize curl.
     *
     * @param array   $config
     * @param boolean $saveCookies
     */
    private function _initCurl(array $config, $saveCookies)
    {
        $url = $this->_configureUrl($config['params']);
        $cookieRequest = $config['set_cookies'];
        $headers = $config['headers'];

        $ch = curl_init();

        // uncomment for debug purposes
        // TODO make sure these are commented out in production code!
        // curl_setopt($ch, CURLOPT_VERBOSE, 1);
        // curl_setopt($ch, CURLINFO_HEADER_OUT, true);

        curl_setopt($ch, CURLOPT_RETURNTRANSFER, 1);
        curl_setopt($ch, CURLOPT_TIMEOUT, 200);
        curl_setopt($ch, CURLOPT_FOLLOWLOCATION, true);
        curl_setopt($ch, CURLOPT_AUTOREFERER, 1);
        curl_setopt($ch, CURLOPT_SSL_VERIFYPEER, false);
        curl_setopt($ch, CURLOPT_URL, $url);

        if ($headers) {
            $headers = $this->_formatHeaders($headers);
            curl_setopt($ch, CURLOPT_HTTPHEADER, $headers);
        }

        if ($cookieRequest) {
            curl_setopt($ch, CURLOPT_COOKIEFILE, self::AB_COOKIEDIR);
        }

        if ($saveCookies) {
            curl_setopt($ch, CURLOPT_COOKIEJAR, self::AB_COOKIEDIR);
            curl_setopt($ch, CURLOPT_COOKIESESSION, true);
        }

        $this->_cURL = $ch;
    }

    /**
     * Configure api url to include parameters.
     *
     * @param string $params
     * @return string
     */
    private function _configureUrl($params)
    {
        $url = $this->_baseUrl;

        if ($params && $this->_isString($params)) {
            $url .= $this->_trimmedParams($params);
        }

        return $url;
    }

    /**
     * Format HTTP headers.
     *
     * @param array $headers
     * @return array
     */
    private function _formatHeaders(array $headers)
    {
        $formattedHeaders = array();
        foreach ($headers as $key => $value) {
            $formattedHeaders[] = "$key: $value";
        }

        return $formattedHeaders;
    }

    /**
     * sends request to specified api endpoint.
     *
     * @param array   $config
     * @param boolean $saveCookies
     * @return array
     */
    protected function _sendGetRequest(array $config, $saveCookies = false)
    {
        $this->_initCurl($config, $saveCookies);
        $ch = $this->_cURL;

        $res = curl_exec($ch);

        $error = $this->_getResponseError($res, $config['set_cookies']);
        if (!empty($error)) {
            $this->_logError($ch);
            curl_close($ch);
            return $error;
        }

        curl_close($ch);

        $assocArray = true;
        $responseData = json_decode($res, $assocArray);
        return $responseData;
    }

    /**
     * Send POST request.
     *
     * @param array   $config
     * @param boolean $saveCookies
     * @return array
     */
    protected function _sendPostRequest(array $config, $saveCookies = false)
    {
        $this->_initCurl($config, $saveCookies);
        $ch = $this->_cURL;

        $postData = $config['data'];
        $this->_configurePost($postData);

        $res = curl_exec($ch);

        $error = $this->_getResponseError($res, $config['set_cookies']);
        if (!empty($error)) {
            $this->_logError();
            curl_close($ch);
            return $error;
        }

        curl_close($ch);

        $assocArray = true;
        $responseData = json_decode($res, $assocArray);
        return $responseData;
    }

    /**
     * Log error message.
     */
    private function _logError()
    {
        $ch = $this->_cURL;
        $errorMessage = 'CURL Error: ' . curl_error($ch) . ' Error number: ' . curl_errno($ch);
        // alt. send in curl_getinfo($ch) as second param - will give more info.
        IW::getLog()->error($errorMessage);
    }

    /**
     * Configure cURL POST request.
     *
     * @param mixed $postData
     */
    private function _configurePost($postData)
    {
        if (gettype($postData) === 'array') {
            $postData = json_encode($postData);
        }

        $ch = $this->_cURL;
        curl_setopt($ch, CURLOPT_POST, 1);
        curl_setopt($ch, CURLOPT_POSTFIELDS, $postData);
    }

    /**
     * Send PUT request.
     *
     * @param array   $config
     * @param boolean $saveCookies
     * @return array
     */
    protected function _sendPutRequest(array $config, $saveCookies = false)
    {
        $this->_initCurl($config, $saveCookies);
        $ch = $this->_cURL;

        $putData = $config['data'];
        $this->_configurePut($putData);

        $res = curl_exec($ch);

        $error = $this->_getResponseError($res, $config['set_cookies']);
        if (!empty($error)) {
            $this->_logError($ch);
            curl_close($ch);
            return $error;
        }

        curl_close($ch);

        $assocArray = true;
        $responseData = json_decode($res, $assocArray);
        return $responseData;
    }

    /**
     * Configure cURL for PUT request.
     *
     * @param array $putData
     */
    private function _configurePut(array $putData)
    {
        if (empty($putData)) {
            throw new Exception('ERROR: PUT must include data!');
        }

        if (gettype($putData) === 'array') {
            $putData = json_encode($putData);
        }

        $ch = $this->_cURL;
        curl_setopt($ch, CURLOPT_CUSTOMREQUEST, 'PUT');
        curl_setopt($ch, CURLOPT_POSTFIELDS, $putData);
    }

    /**
     * Get response error, if any.
     *
     * @param string  $res
     * @param boolean $cookieRequest
     * @return mixed
     */
    private function _getResponseError($res, $cookieRequest)
    {
        if ($this->_requestFailed($res)) {
            $error = $this->_getErrorObject(500, 'Client failed to send request/ response not received.');
            return $error;
        }

        $code = curl_getinfo($this->_cURL, CURLINFO_HTTP_CODE);

        if ($this->_cookieResponseError($cookieRequest, $code)) {
            $error = $this->_getErrorObject($code, 'Cookie request failed: ' . $res);
            return $error;
        }
    }

    /**
     * Format error object if error is thrown.
     *
     * @param integer $code
     * @param string  $msg
     * @return array
     */
    private function _getErrorObject($code, $msg)
    {
        return array(
            'error' => array(
                'code' => $code,
                'message' => $msg
            )
        );
    }

    /**
     * If response failed to send.
     *
     * @param mixed $res
     * @return boolean
     */
    private function _requestFailed($res)
    {
        return $res === false;
    }

    /**
     * Validate cookie requests.
     * Cookie request endpoints do not return json object.
     *
     * @param boolean $cookieRequest
     * @param integer $responseCode
     */
    private function _cookieResponseError($cookieRequest, $responseCode)
    {
        if ($cookieRequest) {
            return $responseCode >= 300;
        }

        return false;
    }

    /**
     * Check if cookies exists.
     *
     * @return boolean
     */
    public function sessionSet()
    {
        return is_file(self::AB_COOKIEDIR);
    }

    /**
     * Check if value is string.
     *
     * @param string $params
     * @return boolean
     */
    private function _isString($params)
    {
        return gettype($params) === 'string';
    }

    /**
     * Remove / at the beginning of params if included, as apiUrl will end with /.
     *
     * @param string $params
     * @return string
     */
    private function _trimmedParams($params)
    {
        if (substr($params, 0) === '/') {
            return ltrim($params, '/');
        }

        return $params;
    }
}
