<?php

/**
 * InterWorx Hosting Control Panel
 *
 * <pre>
 * +----------------------------------------------------------------------+
 * | Copyright (c) 2000-2010 InterWorx L.L.C., All Rights Reserved.       |
 * +----------------------------------------------------------------------+
 * | Redistribution and use in source form, with or without modification  |
 * | is NOT permitted without consent from the copyright holder.          |
 * |                                                                      |
 * | THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS "AS IS" AND |
 * | ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,    |
 * | THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A          |
 * | PARTICULAR PURPOSE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,    |
 * | EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,  |
 * | PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR   |
 * | PROFITS; OF BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY  |
 * | OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT         |
 * | (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE    |
 * | USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH     |
 * | DAMAGE.                                                              |
 * +----------------------------------------------------------------------+
 * </pre>
 *
 * @package InterWorx
 * @author  Malin Lorch <mlorch@liquidweb.com>
 * @date    Apr 02, 2020
 */

require_once(__DIR__ . '/Interface.php');
require_once(__DIR__ . '/Client.php');

/**
 * Acronis Backup Cloud api.
 *
 * @package InterWorx
 * @subpackage Plugin_Acronis
 */
class Model_ApiClient_CookieAuth extends Model_ApiClient_Client implements Model_ApiClient_Interface
{
    private $_User;

    /**
     * Constructor.
     *
     * @param Model_Auth_User $User
     */
    public function __construct(Model_Auth_User $User)
    {
        $url = $User->getDatacenterUrl();
        parent::__construct($url);

        $this->_User = $User;
    }

    /**
     * Get tenant id.
     *
     * @return string
     */
    public function getTenantId()
    {
        return $this->_User->getTenantId();
    }

    /**
     * Send authenticated GET request.
     *
     * @param string $apiEndpoint
     * @return array
     */
    public function sendAuthenticatedGetRequest($apiEndpoint)
    {
        $this->_setCookiesIfNeeded();
        $config = $this->_getCookieConfig($apiEndpoint);
        $res = $this->_sendGetRequest($config);

        // will throw exception if cookie expired.
        try {
            $this->_validateResponse($res, $apiEndpoint);
            return $this->_returnItems($res);
        } catch (Exception $e) {
            // reissue new cookie.
            $this->_requestBackupConsoleCookie();
            $res = $this->_sendGetRequest($config);
            $this->_validateResponse($res, $apiEndpoint);
            return $this->_returnItems($res);
        }
    }

    /**
     * Send authenticated POST request.
     *
     * @param string $apiEndpoint
     * @param array  $config
     * @return array
     */
    public function sendAuthenticatedPostRequest($apiEndpoint, array $config = array())
    {
        $this->_setCookiesIfNeeded();
        $config = $this->_getCookieConfig($apiEndpoint, $config);
        $res = $this->_sendPostRequest($config);

        try {
            $this->_validateResponse($res, $apiEndpoint);
            return $this->_returnItems($res);
        } catch (Exception $e) {
            // reissue new cookie.
            $this->_requestBackupConsoleCookie();
            $res = $this->_sendPostRequest($config);
            $this->_validateResponse($res, $apiEndpoint);
            return $this->_returnItems($res);
        }
    }

    /**
     * Send authenticated PUT request.
     *
     * @param string $apiEndpoint
     * @param array  $data
     * @return array
     */
    public function sendAuthenticatedPutRequest($apiEndpoint, array $data = null)
    {
        $this->_setCookiesIfNeeded();
        $config = $this->_getCookieConfig($apiEndpoint, $data);
        $res = $this->_sendPutRequest($config);

        try {
            $this->_validateResponse($res, $apiEndpoint);
            return $this->_returnItems($res);
        } catch (Exception $e) {
            // reissue new cookie.
            $this->_requestBackupConsoleCookie();
            $res = $this->_sendPutRequest($config);
            $this->_validateResponse($res, $apiEndpoint);
            return $this->_returnItems($res);
        }
    }

    /**
     * Send request to receive & save cookies if cookies file is missing.
     */
    private function _setCookiesIfNeeded()
    {
        // Will only detect if no cookies have been set yet.
        // Does not detect expired cookies.
        if (!$this->sessionSet()) {
            $this->_requestBackupConsoleCookie();
        }
    }

    /**
     * Get config array for cookie request.
     *
     * @param string $apiEndpoint
     * @param array  $config
     * @return array
     */
    private function _getCookieConfig($apiEndpoint, array $config = array())
    {
        $customHeaders = $config['headers'];
        $headers =  !empty($customHeaders) ? $customHeaders : $this->_getDefaultHeaders();
        $postData = $config['data'];

        $postConfig = array(
            'params' => $apiEndpoint,
            'set_cookies' => true,
            'headers' => $headers
        );

        if (!empty($postData)) {
            $postConfig['data'] = $postData;
        }

        return $postConfig;
    }

    /**
     * Get default headers.
     *
     * @return array
     */
    private function _getDefaultHeaders()
    {
        return array(
            'Accept' => 'application/json',
            'Content-type' => 'application/json'
        );
    }

    /**
     * Validate response object
     *
     * @param mixed  $response
     * @param string $apiEndpoint
     * @throws Exception
     */
    private function _validateResponse($response, $apiEndpoint)
    {
        if (is_array($response) && isset($response['error'])) {
            $responseError = $response['error'];
            $errorMessage = $responseError['message'];
            $responseCode = $responseError['code'];
            throw new Exception_AcronisApiFailure("ERROR CookieAuth Client. Api endpoint: $apiEndpoint. Response code: $responseCode. Error message: $errorMessage");
        }
    }

    /**
     * Format response object to return items/data.
     *
     * @param mixed $res
     * @return array
     */
    private function _returnItems($res)
    {
        if (isset($res['data'])) {
            return $res['data'];
        }

        if (isset($res['items'])) {
            return $res['items'];
        }

        return $res;
    }

    /**
     * Save cookies.
     *
     * @throws Exception
     */
    private function _requestBackupConsoleCookie()
    {
        $token = $this->_requestBackupConsoleAccessToken();
        $headers = $this->_basicHeaders();
        $data = array("access_token" => $token);
        $apiEndpoint = '/bc/api/remote_connection';

        $config = array(
            'headers' => $headers,
            'data' => $data,
            'params' => $apiEndpoint
        );

        // no point in returning response - curl will save cookies.
        $saveCookies = true;
        $res = $this->_sendPostRequest($config, $saveCookies);
        $this->_validateResponse($res, $apiEndpoint);
    }

    /**
     * Request access tken used for cookie authentication.
     *
     * @return string
     */
    private function _requestBackupConsoleAccessToken()
    {
        $tenantId = $this->_User->getTenantId();
        $headers = $this->_basicHeaders();
        $apiEndpoint = "/api/1/groups/$tenantId/backupconsole";

        $config = array(
            'headers' => $headers,
            'params' => $apiEndpoint
        );

        $res = $this->_sendGetRequest($config);
        $this->_validateResponse($res, $apiEndpoint);
        return $res['token'];
    }

    /**
     * Get headers for basic authentication.
     *
     * @return array
     */
    private function _basicHeaders()
    {
        $basic = $this->_User->getBasicCredentials();
        $headers = array(
            'Authorization' => "Basic $basic",
            'Accept' => 'application/json',
            'Content-type' => 'application/json'
        );
        // TODO remove content-type for certain requests.

        return $headers;
    }
}
