<?php

/**
 * InterWorx Hosting Control Panel
 *
 * <pre>
 * +----------------------------------------------------------------------+
 * | Copyright (c) 2000-2010 InterWorx L.L.C., All Rights Reserved.       |
 * +----------------------------------------------------------------------+
 * | Redistribution and use in source form, with or without modification  |
 * | is NOT permitted without consent from the copyright holder.          |
 * |                                                                      |
 * | THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS "AS IS" AND |
 * | ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,    |
 * | THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A          |
 * | PARTICULAR PURPOSE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,    |
 * | EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,  |
 * | PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR   |
 * | PROFITS; OF BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY  |
 * | OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT         |
 * | (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE    |
 * | USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH     |
 * | DAMAGE.                                                              |
 * +----------------------------------------------------------------------+
 * </pre>
 *
 * @package InterWorx
 * @author  Malin Lorch <mlorch@liquidweb.com>
 * @date    Nov 27, 2019
 */

require_once(__DIR__ . '/Interface.php');
require_once(__DIR__ . '/Client.php');
require_once(__DIR__ . '/../../Exception/InvalidCredentials.php');
require_once(__DIR__ . '/../../Exception/InvalidRequestConfig.php');

/**
 * Acronis api for authorization.
 *
 * @package Interworx
 * @subpackage Plugin_Acronis
 */
class Model_ApiClient_OAuth extends Model_ApiClient_Client implements Model_ApiClient_Interface
{
    const TOKEN_EXPIRED = 'OAuth Token Expired';
    const TOKEN_MISSING = 'OAuth Token Missing';
    const AB_TOKEN_ENV = 'ab_token';

    private $_basicCredentials;
    private $_oauthToken;
    private $_errorMessage;
    private $_apiParams;


    /**
     * Set base api url/credentials and initiate cURL.
     *
     * @param Model_Auth_User $User
     */
    public function __construct(Model_Auth_User $User)
    {
        $url = $User->getDatacenterUrl();
        parent::__construct($url);

        $this->_User = $User;
        $this->_basicCredentials = $User->getBasicCredentials();
        $this->_apiParams = '/api/2/';
        $this->_baseUrl = $url . $this->_apiParams;
    }

    /**
     * get base url for api.
     *
     * @return string
     */
    public function getApiBaseUrl()
    {
        return $this->_baseUrl;
    }

    /**
     * Encrypt username & credentials for api use.
     *
     * @param string $username
     * @param string $password
     * @return string
     */
    public static function encryptCredentials($username, $password)
    {
        return base64_encode("$username:$password");
    }

    /**
     * Check if user is authorized.
     *
     * @return boolean
     */
    public function isAuthorized()
    {
        try {
            $this->_getAuthTokenFromEnv();
            return true;
        } catch (Exception $e) {
            return false;
        }
    }

    /**
     * Authorize user.
     * Sends multiple requests to acronis api to validate user.
     *
     * @param boolean $saveToken
     * @throws Exception
     */
    public function authorizeUser($saveToken = true)
    {
        $basicResponse = $this->_basicAuthRequest();
        $this->_validateResponse($basicResponse, 'Basic Request');

        $clientsResponse = $this->_clientsAuthRequest($basicResponse['tenant_id']);
        $this->_validateResponse($clientsResponse, 'Client Request');

        $clientId = $clientsResponse['client_id'];
        $clientSecret = $clientsResponse['client_secret'];
        $encryptedClient = self::encryptCredentials($clientId, $clientSecret);

        $oauthResponse = $this->_oauthRequest($encryptedClient);
        $this->_validateResponse($oauthResponse, 'OAuth Request');

        // save variables if all went well.
        $this->_User->setTenantId($basicResponse['tenant_id']);
        $this->_User->setUserId($basicResponse['id']);

        if ($saveToken) {
            $this->_saveOAuthTokenData($oauthResponse);
        }
    }

    /**
     * Validate response object.
     *
     * @param object $response
     * @param string $action
     * @throws Exception
     */
    private function _validateResponse($response, $action)
    {
        // code only included in response if error.
        $responseError = $response['error'];
        if ($responseError) {
            $this->_logErrorMessage($responseError, $action);
            throw new Exception_InvalidCredentials();
        }
    }

    /**
     * Log error message.
     *
     * @param array  $responseError
     * @param string $failedAction
     */
    private function _logErrorMessage(array $responseError, $failedAction)
    {
        $responseCode = $responseError['code'];
        $errorMsg = $responseError['message'];
        IW::getLog()->error("Action failed: $failedAction - response code $responseCode: $errorMsg");
    }

    /**
     * Send request for basic validation, e.g. username and password.
     *
     * @return array
     */
    private function _basicAuthRequest()
    {
        $config = $this->_getBasicValidationRequestConfig($this->_basicCredentials);
        return $this->_sendGetRequest($config);
    }

    /**
     * Send request for client id & client secret.
     *
     * @param string $tenantId
     * @return array
     */
    private function _clientsAuthRequest($tenantId)
    {
        $config = $this->_getClientRequestConfig($tenantId);
        return $this->_sendPostRequest($config);
    }

    /**
     * Send request for OAuth validation.
     *
     * @param string $clientCredentials
     * @return array
     */
    private function _oauthRequest($clientCredentials)
    {
        $config = $this->_getOAuthRequestConfig($clientCredentials);
        return $this->_sendPostRequest($config);
    }

    /**
     * Request configuration for basic validation.
     *
     * @param string $credentials
     * @return array
     */
    private function _getBasicValidationRequestConfig($credentials)
    {
        $headers = array(
            'Authorization' => "Basic $credentials"
        );

        $config = array(
            'headers' => $headers,
            'params' => $this->_apiParams . 'users/me'
        );

        return $config;
    }

    /**
     * Request configuration for client authorization.
     *
     * @param string $tenantId
     * @return array
     */
    private function _getClientRequestConfig($tenantId)
    {
        $data = array(
            'agent_type' => 'backupAgent',
            'hostname' => 'ag1Host',
            'client_name' => 'C2C backup agent'
        );

        $postData = array(
            'type' => 'agent',
            'tenant_id' => $tenantId,
            'data' => $data,
            'token_endpoint_auth_method' => 'client_secret_basic'
        );

        $headers = array(
            'Authorization' => "Basic $this->_basicCredentials",
            'Content-type' => 'application/json',
            'Accept' => 'application/json'
        );

        $config = array(
            'headers' => $headers,
            'params' => $this->_apiParams . 'clients',
            'data' => $postData
        );

        return $config;
    }

    /**
     * Request configuraion for obtaining oAuth token.
     *
     * @param string $clientCredentials
     * @return array
     */
    private function _getOAuthRequestConfig($clientCredentials)
    {
        $headers = array(
            'Accept' => 'application/json',
            'Authorization' => "Basic $clientCredentials"
        );

        $data = array(
            'grant_type' => 'client_credentials',
            'scope' => 'urnacronis.comaccount-serverroot_admin'
        );

        $postData = http_build_query($data);

        $config = array(
            'headers' => $headers,
            'data' => $postData,
            'params' => $this->_apiParams . 'idp/token'
        );

        return $config;
    }

    /**
     * Save encrypted token as env flag.
     *
     * @param object $tokenData
     */
    private function _saveOauthTokenData($tokenData)
    {
        $encryptedToken = IWorxEncrypt::encrypt($tokenData['access_token']);

        $data = array(
            'access_token' => $encryptedToken,
            'expires_on' => $tokenData['expires_on']
        );

        $json = json_encode($data);
        IW::Env()->setEnvFlag(self::AB_TOKEN_ENV, $json);
    }

    /**
     * Get decrypted token from env flag.
     *
     * @return string
     * @throws Exception
     */
    private function _getOAuthToken()
    {
        if (!$this->_oauthToken) {
            try {
                $this->_oauthToken = $this->_getAuthTokenFromEnv();
            } catch (Exception $e) {
                // generate new token
                $this->authorizeUser();
                $this->_oauthToken = $this->_getAuthTokenFromEnv();
            }
        }

        return $this->_oauthToken;
    }

    /**
     * Get OAuth token from env if previously stored.
     * Will throw exception if token is invalid or expired.
     *
     * @return string
     * @throws Exception
     */
    private function _getAuthTokenFromEnv()
    {
        $jsonData = IW::Env()->getEnvFlag(self::AB_TOKEN_ENV);

        if (empty($jsonData)) {
            throw new Exception(self::TOKEN_MISSING);
        }

        $tokenData = json_decode($jsonData);

        $expirationTime = $tokenData->expires_on;
        $currentTime = time();

        if ($expirationTime < $currentTime) {
            throw new Exception(self::TOKEN_EXPIRED);
        }

        return IWorxEncrypt::decrypt($tokenData->access_token);
    }

    /**
     * Delete OAuth token from env.
     */
    public function clearAuth()
    {
        IW::Env()->clearEnvFlag(self::AB_TOKEN_ENV);
    }

    /**
     * Get error message.
     *
     * @return string error message
     */
    public function getErrorMessage()
    {
        return $this->_errorMessage;
    }

    /**
     * Tenant id saved when authorizing user.
     *
     * @return string
     */
    public function getTenantId()
    {
        return $this->_User->getTenantId();
    }

    /**
     * Return user id.
     *
     * @return string
     */
    public function getUserId()
    {
        $User = $this->_User;

        $userIdNotSet = $User->getUserId() === null;
        if ($userIdNotSet) {
            $this->authorizeUser();
        }

        return $User->getUserId();
    }

    /**
     * Return username.
     *
     * @return string
     */
    public function getUsername()
    {
        return $this->_User->getUsername();
    }

    /**
     *
     *
     * @return string
     */
    public function getDatacenterUrl()
    {
        return $this->_User->getDatacenterUrl();
    }

    /**
     * Send request with oauth token.
     *
     * @param array $config
     * @return array
     */
    public function sendAuthenticatedRequest(array $config)
    {
        if (empty($config['params'])) {
            throw new Exception_InvalidRequestConfig('params for api missing!');
        }

        $headers = $this->_getAuthenticatedHeaders($config);
        $requestConfig = array(
            'params' => $config['params'],
            'headers' => $headers
        );

        if (isset($config['data'])) {
            $requestConfig['data'] = $config['data'];
            return $this->_sendPostRequest($requestConfig);
        }

        return $this->_sendGetRequest($requestConfig);
    }

    /**
     * Get headers with oauth authentication.
     *
     * @param array $config
     * @return array
     */
    private function _getAuthenticatedHeaders(array $config)
    {
        $authToken = $this->_getOAuthToken();
        $configHeaders = $config['headers'];

        if (isset($configHeaders)) {
            $configHeaders['Authorization'] = "Bearer $authToken";
            return $configHeaders;
        }

        return array('Authorization' => "Bearer $authToken");
    }
}
