<?php
/**
 * InterWorx Hosting Control Panel
 *
 * <pre>
 * +----------------------------------------------------------------------+
 * | Copyright (c) 2000-2010 InterWorx L.L.C., All Rights Reserved.       |
 * +----------------------------------------------------------------------+
 * | Redistribution and use in source form, with or without modification  |
 * | is NOT permitted without consent from the copyright holder.          |
 * |                                                                      |
 * | THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS "AS IS" AND |
 * | ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,    |
 * | THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A          |
 * | PARTICULAR PURPOSE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,    |
 * | EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,  |
 * | PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR   |
 * | PROFITS; OF BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY  |
 * | OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT         |
 * | (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE    |
 * | USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH     |
 * | DAMAGE.                                                              |
 * +----------------------------------------------------------------------+
 * </pre>
 *
 * @package InterWorx
 * @author  Malin Lorch <mlorch@liquidweb.com>
 * @date    Apr 21, 2020
 */

/**
 *
 *
 * @package Interworx
 * @subpackage Plugin_Acronis
 */
class Model_Auth_Api extends Model_Api
{
    private $_OAuthClient;

    /**
     * Constructor.
     */
    public function __construct()
    {
        $this->_OAuthClient = $this->_getOAuthClient();
    }

    /**
     * Authorize user.
     */
    public function authorizeUser()
    {
        if (!$this->_OAuthClient->isAuthorized()) {
            $this->_OAuthClient->authorizeUser();
        }
    }

    /**
     * Get one-time login link that expires after 30 seconds.
     */
    public function getOneTimeLoginLink()
    {
        $oneTimeToken = $this->_requestOtt();
        $redirectUrl = $this->_getOneTimeLoginUrl($oneTimeToken);
        return $redirectUrl;
    }

    /**
     * Request one-time-token.
     *
     * @return string
     */
    private function _requestOtt()
    {
        $config = $this->_getOttConfig();
        $response = $this->_OAuthClient->sendAuthenticatedRequest($config);

        $this->_handleOttResponseError($response);
        return $response['ott'];
    }


    /**
     * Get config array for ott request.
     *
     * @return array
     */
    private function _getOttConfig()
    {
        $ottData = $this->_getOttData();
        $headers = $this->_getOttHeaders();

        return array(
            'headers' => $headers,
            'data' => $ottData,
            'params' => '/api/2/idp/ott'
        );
    }

    /**
     * Get ott data.
     *
     * @return array
     */
    private function _getOttData()
    {
        $username = $this->_OAuthClient->getUsername();

        return array(
            'login' => $username,
            'purpose' => 'user_login'
        );
    }

    /**
     * Get ott headers.
     *
     * @return array
     */
    private function _getOttHeaders()
    {
        return array(
            'Content-type' => 'application/json',
            'Accept' => 'application/json'
        );
    }

    /**
     * Handle ott response error.
     *
     * @param array $response
     */
    private function _handleOttResponseError(array $response)
    {
        $responseError = $response['error'];
        if ($responseError) {
            $responseCode = $responseError['code'];
            $responseMessage = $responseError['message'];
            throw new Exception("Ott request failed: $responseCode: $responseMessage");
        }
    }

    /**
     * Get one-time login url.
     *
     * @param string $ott
     * @return string
     */
    private function _getOneTimeLoginUrl($ott)
    {
        $datacenterUrl = $this->_OAuthClient->getDatacenterUrl();
        $apiUrl = $this->_OAuthClient->getApiBaseUrl();
        $query = $this->_getOttQuery($ott);
        $redirectUrl =  $apiUrl . "idp/external-login#$query&targetURI=$datacenterUrl/bc";
        return $redirectUrl;
    }

    /**
     * Get ott query string.
     *
     * @param string $oneTimeToken
     * @return string
     */
    private function _getOttQuery($oneTimeToken)
    {
        $config = array('ott' => $oneTimeToken);
        return http_build_query($config);
    }
}
