<?php

/**
 * InterWorx Hosting Control Panel
 *
 * <pre>
 * +----------------------------------------------------------------------+
 * | Copyright (c) 2000-2010 InterWorx L.L.C., All Rights Reserved.       |
 * +----------------------------------------------------------------------+
 * | Redistribution and use in source form, with or without modification  |
 * | is NOT permitted without consent from the copyright holder.          |
 * |                                                                      |
 * | THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS "AS IS" AND |
 * | ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,    |
 * | THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A          |
 * | PARTICULAR PURPOSE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,    |
 * | EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,  |
 * | PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR   |
 * | PROFITS; OF BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY  |
 * | OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT         |
 * | (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE    |
 * | USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH     |
 * | DAMAGE.                                                              |
 * +----------------------------------------------------------------------+
 * </pre>
 *
 * @package InterWorx
 * @author  Malin Lorch <mlorch@liquidweb.com>
 * @date    Apr 22, 2020
 */

/**
 *
 *
 * @package Interworx
 * @subpackage Plugin_Acronis
 */
class Model_BackupAgent_BackupAgent extends Model_Acronis
{
    const PATH_TMP_INSTALL = '/tmp/Backup_Agent_Linux_x86_64.bin';
    const CREDENTIALS_ENV = 'acroniscredentials';
    const MMS_FILE_PATH = '/usr/sbin/acronis_mms';

    const ACRONIS_DIR = '/var/lib/Acronis';
    const AGENTCOMMDATA_PATH = self::ACRONIS_DIR . '/AgentCommData';

    const PRE_SCRIPT_PATH = self::AGENTCOMMDATA_PATH . '/pre-capture-data.sh';
    const POST_SCRIPT_PATH = self::AGENTCOMMDATA_PATH . '/post-capture-data.sh';
    const MYSQL_SCRIPT_PATH = self::AGENTCOMMDATA_PATH . '/create-mysql-dumps.sh';
    const SCRIPT_CONFIG_PATH = self::AGENTCOMMDATA_PATH . '/capture-data-config.sh';

    const PLUGIN_SCRIPTS_SOURCE_PATH = __DIR__ . '/../../scripts/AgentCommData';


    /**
     * Uninstall backup agent.
     */
    public function uninstall()
    {
        if (!$this->isInstalled()) {
            throw new Exception_BackupAgentUninstallError('Backup agent not installed!');
        }
        $this->_uninstallBackupAgent();

        $this->_clearEnvIfExists(self::CREDENTIALS_ENV);
        $this->_clearEnvIfExists(Model_ApiClient_OAuth::AB_TOKEN_ENV);

        $this->_getCache()->deleteAll();
    }

    /**
     * Uninstall backup agent.
     */
    private function _uninstallBackupAgent()
    {
        $execInstallScript = '/usr/lib/Acronis/BackupAndRecovery/uninstall/uninstall --auto';

        $Command = new Command($execInstallScript);
        $Command->runAsRoot();
        $Command->exec();

        if (!$Command->wasSuccessful()) {
            throw new Exception_BackupAgentUninstallError('unable to uninstall acronis backup agent...');
        }
    }



    /**
     * Install backup agent.
     *
     * @throws Exception
     */
    public function install()
    {
        if ($this->isInstalled()) {
            throw new Exception_BackupAgentInstallFailure('Backup agent already intalled!');
        }

        $this->_removeInstallScriptIfNeeded();
        $this->_downloadInstallScript();
        $this->_makeFileExecutable();

        $this->_runInstallationScript();
        $this->addPrePostScripts();
        $this->_installIwCli();
    }

    /**
     * Remove old install script if needed.
     */
    private function _removeInstallScriptIfNeeded()
    {
        if ($this->_isFile(self::PATH_TMP_INSTALL)) {
            $removeOldFile = 'rm -rf ' . self::PATH_TMP_INSTALL;
            $Command = new Command($removeOldFile);
            $Command->runAsRoot();
            $Command->exec();

            if (!$Command->wasSuccessful()) {
                throw new Exception_BackupAgentInstallFailure('unable to remove old file');
            }
        }
    }

    /**
     * Download acronis installation file.
     * Needs acronis user specific url with login id.
     */
    private function _downloadInstallScript()
    {
        $login = $this->_getApi()->getUserId();

        $dataCenterUrl =  $this->_getAcronisUser()->getDataCenterUrl();
        $scriptUrl = $this->_getDownloadScriptUrl($dataCenterUrl, $login);
        $this->_runDownloadInstallFileCommand($scriptUrl);
    }

    /**
     * Get acronis user.
     */
    private function _getAcronisUser()
    {
        $Plugin = $this->_getPlugin();
        return $Plugin->getAcronisUser();
    }

    /**
     * Builds url for downloading acronis installation script.
     * Url & login/userid is specific to acronis account.
     *
     * @param string $url
     * @param string $login
     * @return string
     */
    private function _getDownloadScriptUrl($url, $login)
    {
        $params = '/bc/api/ams/links/agents/redirect?language=multi&system=linux&architecture=64&productType=enterprise&login=';
        $url = $url . $params . $login;
        return $url;
    }

    /**
     * Runs wget command to download installation file.
     *
     * @param string $scriptUrl
     */
    private function _runDownloadInstallFileCommand($scriptUrl)
    {
        $downloadFile = 'wget -O ' . self::PATH_TMP_INSTALL . ' "' . $scriptUrl . '"';
        $Command = new Command($downloadFile);
        $Command->allowQuoteDouble();
        $Command->allowAmpersand();
        $Command->allowColon();
        $Command->runAsRoot();
        $Command->exec();
    }

    /**
     * Update file permissions to allow execution.
     */
    private function _makeFileExecutable()
    {
        $makeFileExecutable = 'chmod 0755 ' . self::PATH_TMP_INSTALL;
        $Command = new Command($makeFileExecutable);

        $Command->runAsRoot();
        $Command->exec();

        if (!$Command->wasSuccessful()) {
            throw new Exception_BackupAgentInstallFailure('Unable to update file permissions. ' . $Command->getResultMessage());
        }
    }

    /**
     * Execute installation script.
     */
    private function _runInstallationScript()
    {
        $runInstallationScript = $this->_getInstallCommand();
        $Command = new Command($runInstallationScript);

        $Command->allowColon();

        // in case password or username contains special characters.
        $Command->allowAmpersand();
        $Command->allowAsterisk();
        $Command->allowBracketBoth();
        $Command->allowCaret();
        $Command->allowCurlyBoth();
        $Command->allowDollarSign();
        $Command->allowParenthesisBoth();
        $Command->allowPound();
        $Command->allowQuestionMark();
        $Command->allowRedirectIn();
        $Command->allowRedirectOut();
        $Command->allowSemicolon();


        $Command->runAsRoot();
        $Command->exec();

        if (!$Command->wasSuccessful()) {
            $Command->logResult();
            throw new Exception_BackupAgentInstallFailure('Failed executing installation script');
        }
    }


    /**
     * Get command for running the installation file.
     *
     * @return string
     */
    private function _getInstallCommand()
    {
        $AcronisUser = $this->_getAcronisUser();

        $url = $AcronisUser->getDatacenterUrl();
        $username = $AcronisUser->getUsername();
        $password = $AcronisUser->getPassword();


        // --auto - for noninteractive mode
        // --strict - converts all warnings to errors
        // --rain - backup service url
        $cmd = self::PATH_TMP_INSTALL . " --auto --strict --rain $url --login $username --password $password";
        return $cmd;
    }


    /**
     * Add pre & post scripts if they don't exist.
     */
    public function addPrePostScriptsIfMissing()
    {
        if ($this->scriptsMissing()) {
            $this->addPrePostScripts();
        }
    }

    /**
     * add pre post scripts.
     */
    public function addPrePostScripts()
    {
        $srcScripts = __DIR__ . '/../../scripts/AgentCommData';
        $destination = self::ACRONIS_DIR;
        $agentCommDataPath = self::AGENTCOMMDATA_PATH;

        $copyScriptsToAcronisLocation = "cp -a $srcScripts $destination && chmod -R 750 $agentCommDataPath";

        $Command = new Command($copyScriptsToAcronisLocation);
        $Command->allowAmpersand();
        $Command->runAsRoot();
        $Command->exec();

        if ($this->scriptsMissing()) {
            throw new Exception_BackupAgentInstallFailure('Failed to copy scripts to ' . self::ACRONIS_DIR);
        }
    }

    /**
     * Clear env flag if file exists.
     *
     * @param string $envFlag
     */
    private function _clearEnvIfExists($envFlag)
    {
        $base = Ini::get(Ini::IWORX_DIR, 'base');
        if ($this->_isFile("$base/etc/env/$envFlag")) {
            IW::Env()->clearEnvFlag($envFlag);
        }
    }

    /**
     * Check if acronis is installed by verifying file existance.
     *
     * @return boolean
     */
    public function isInstalled()
    {
        return  $this->_isFile(self::MMS_FILE_PATH);
    }

    /**
     * check if scripts are missing.
     *
     * @return boolean
     */
    public function scriptsMissing()
    {
        $paths = array(self::PRE_SCRIPT_PATH, self::POST_SCRIPT_PATH, self::SCRIPT_CONFIG_PATH, self::MYSQL_SCRIPT_PATH);
        foreach ($paths as $path) {
            if (!$this->_isFile($path)) {
                return true;
            }
        }

        return false;
    }

    /**
     * used in pre backup scripts.
     */
    private function _installIwCli()
    {
        $cmd = 'yum install interworx-cli -y';
        $Command = new Command($cmd);
        $Command->runAsRoot();
        $Command->exec();
        if (!$Command->wasSuccessful()) {
            throw new Exception_BackupAgentInstallFailure('unable to install interworx-cli');
        }
    }
}
