<?php

/**
 * InterWorx Hosting Control Panel
 *
 * <pre>
 * +----------------------------------------------------------------------+
 * | Copyright (c) 2000-2010 InterWorx L.L.C., All Rights Reserved.       |
 * +----------------------------------------------------------------------+
 * | Redistribution and use in source form, with or without modification  |
 * | is NOT permitted without consent from the copyright holder.          |
 * |                                                                      |
 * | THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS "AS IS" AND |
 * | ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,    |
 * | THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A          |
 * | PARTICULAR PURPOSE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,    |
 * | EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,  |
 * | PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR   |
 * | PROFITS; OF BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY  |
 * | OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT         |
 * | (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE    |
 * | USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH     |
 * | DAMAGE.                                                              |
 * +----------------------------------------------------------------------+
 * </pre>
 *
 * @package InterWorx
 * @author  Malin Lorch <mlorch@liquidweb.com>
 * @date    Apr 20, 2020
 */

/**
 * Backup Plan class.
 *
 * @package Interworx
 * @subpackage Plugin_Acronis
 */
class Model_Backups_BackupPlan extends Model_Acronis
{
    private $_id;
    private $_type;
    private $_enabled;
    private $_scheduledDays;
    private $_scheduledTime;
    private $_prePostCommands;
    private $_preCommands;
    private $_postCommands;
    private $_planDetails;

    /**
     * Constructor.
     *
     * @param string $id
     */
    public function __construct($id)
    {
        $this->_id = $id;
    }


    /**
     * Get plan details as array.
     *
     * @return array
     */
    public function getPlanDetails()
    {
        if (!isset($this->_planDetails)) {
            $this->_setPlan();
        }

        return $this->_formatPlanDetails($this->_planDetails);
    }

    /**
     * @param array $details
     * @return array
     */
    private function _formatPlanDetails(array $details)
    {
        return array(
            'enabled' => $details['enabled'],
            'id' => $details['id'],
            'time' => $details['time'],
            'days' => $details['days']
        );
    }

    /**
     * Check if plan is enabled.
     *
     * @return boolean
     */
    public function isEnabled()
    {
        if (!isset($this->_enabled)) {
            $this->_setPlan();
        }

        return $this->_enabled;
    }

    /**
     * Get scheduled days.
     *
     * @return array
     */
    public function getScheduledDays()
    {
        if (!isset($this->_scheduledDays)) {
            $this->_setPlan();
        }

        return $this->_scheduledDays;
    }

    /**
     * Get scheduled time.
     *
     * @return array
     */
    public function getScheduledTime()
    {
        if (!isset($this->_scheduledTIme)) {
            $this->_setPlan();
        }

        return $this->_scheduledTime;
    }

    /**
     * Get type.
     *
     * @return string
     */
    public function getType()
    {
        if (!isset($this->_type)) {
            $this->_setPlan();
        }

        return $this->_type;
    }

    /**
     * Get id.
     *
     * @return string
     */
    public function getId()
    {
        return $this->_id;
    }


    /**
     * Check if pre/post scripts are ok.
     *
     * @return boolean
     */
    public function prePostScriptsOK()
    {
        if (!isset($this->_prePostCommands)) {
            $this->_setPlan();
        }

        return $this->_prePostScriptsOK();
    }

    /**
     * Check if pre/post scripts are ok.
     */
    private function _prePostScriptsOK()
    {
        $commandsEnabled = $this->_prePostCommandsEnabled();
        $preCommandConfigOK = $this->_preCommandConfigOK($this->_preCommands);
        $postCommandConfigOK = $this->_postCommandConfigOK($this->_postCommands);

        return $commandsEnabled && $preCommandConfigOK && $postCommandConfigOK;
    }

    /**
     * Check if pre/post commands are enabled.
     *
     * @return boolean
     */
    private function _prePostCommandsEnabled()
    {
        return $this->_prePostCommands['usePreCommands'] && $this->_prePostCommands['usePostCommands'];
    }

    /**
     * Check if existing plan has right configuration.
     *
     * @param array $commandConfig
     * @return boolean
     */
    private function _preCommandConfigOK(array $commandConfig)
    {
        $expectedConfig = array(
            'workingDirectory' => '',
            'continueOnCommandError' => false,
            'commandArguments' => '{RESOURCE_ID}',
            'waitCommandComplete' => true,
            'command' => '/usr/lib/Acronis/BackupAndRecovery/webcpprecapture'
        );

        foreach ($expectedConfig as $key => $value) {
            if ($commandConfig[$key] !== $value) {
                return false;
            }
        }

        return true;
    }

    /**
     * Check if existing plan has right configuration.
     *
     * @param array $commandConfig
     * @return boolean
     */
    private function _postCommandConfigOK(array $commandConfig)
    {
        $expectedConfig = array(
            'workingDirectory' => '',
            'continueOnCommandError' => false,
            'commandArguments' => '{RESOURCE_ID}',
            'waitCommandComplete' => true,
            'command' => '/usr/lib/Acronis/BackupAndRecovery/webcppostcapture'
        );

        foreach ($expectedConfig as $key => $value) {
            if ($commandConfig[$key] !== $value) {
                return false;
            }
        }

        return true;
    }




    /**
     * Set plan.
     */
    private function _setPlan()
    {
        $plan = $this->_getCache()->getPlan();

        if (!isset($plan)) {
            $plan = $this->_requestExistingPlan();
        }

        $this->_setPlanDetails($plan);
    }

    /**
     * Request plan from api. Will throw exception if plan does not exist.
     * Api will throw exception if plan does not exist
     *
     * @return array
     */
    private function _requestExistingPlan()
    {
        $plan = $this->_getApi()->getPlanById($this->_id);

        if (empty($plan)) {
            $this->_getCache()->deletePlanId();
            throw new Exception_BackupPlanMissing('Backup plan does not exist! ' . $this->_id);
        }

        return $this->_formatApiResponse($plan);
    }

    /**
     * format plan from api response.
     *
     * @param array $plan
     */
    private function _formatApiResponse(array $plan)
    {
        $scheme = $plan['scheme'];
        $schedule = $scheme['schedule'];

        $options = $plan['options'];
        $backupOptions = $options['backupOptions'];

        $prePostCommands = $backupOptions['prePostCommands'];
        $preCommand = $prePostCommands['preCommands'];
        $postCommand = $prePostCommands['postCommands'];


        return array(
            'id' => $plan['id'],
            'type' => $plan['type'],
            'enabled' => $plan['enabled'],
            'days' => $schedule['daysOfWeek'],
            'time' => $schedule['startAt'],
            'prePostCommands' => $prePostCommands,
            'preCommands' => $preCommand,
            'postCommands' => $postCommand
        );
    }

    /**
     * Set plan details.
     *
     * @param array $plan
     */
    private function _setPlanDetails(array $plan)
    {
        $this->_type = $plan['type'];
        $this->_enabled = $plan['enabled'];
        $this->_scheduledDays = $plan['days'];
        $this->_scheduledTime = $plan['time'];
        $this->_prePostCommands = $plan['prePostCommands'];
        $this->_preCommands = $plan['preCommands'];
        $this->_postCommands = $plan['postCommands'];
        $this->_planDetails = $plan;
    }

    /**
     * Cache plan.
     */
    public function cachePlan()
    {
        $this->_getCache()->cachePlan($this->_planDetails);
    }

    /**
     * Throw exception if plan does not exist.
     *
     * @param array $plan
     */
    private function _throwExceptionIfPlanDoesNotExist(array $plan)
    {
        if (empty($plan)) {
            $this->_throwExceptionPlanDoesNotExist();
        }
    }
    /**
     * Throw exception if plan does not exist.
     */
    private function _throwExceptionPlanDoesNotExist()
    {
        throw new Exception('Plan does not exist!');
    }

    /**
     * Add resource to plan.
     *
     * @param string $resourceId
     */
    public function addResource($resourceId)
    {
        if (!isset($this->_planDetails)) {
            $this->_setPlan();
        }

        if (!$this->_prePostScriptsOK()) {
            throw new Exception_BackupPlanIncompatible('Plan does not have pre/post scripts enabled!');
        }

        $this->_getApi()->addResourceToPlan($this->_id, $resourceId);
    }

    /**
     * Remove resource from plan.
     *
     * @param string $resourceId
     */
    public function removeResource($resourceId)
    {
        if (!isset($this->_planDetails)) {
            $this->_setPlan();
        }

        $this->_getApi()->removeResourceFromPlan($this->_id, $resourceId);
    }

    /**
     * Run backup.
     *
     * @param string $resourceId
     */
    public function runBackup($resourceId)
    {
        $this->_getApi()->runBackup($this->_id, $resourceId);
    }
}
