<?php
/**
 * InterWorx Hosting Control Panel
 *
 * <pre>
 * +----------------------------------------------------------------------+
 * | Copyright (c) 2000-2010 InterWorx L.L.C., All Rights Reserved.       |
 * +----------------------------------------------------------------------+
 * | Redistribution and use in source form, with or without modification  |
 * | is NOT permitted without consent from the copyright holder.          |
 * |                                                                      |
 * | THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS "AS IS" AND |
 * | ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,    |
 * | THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A          |
 * | PARTICULAR PURPOSE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,    |
 * | EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,  |
 * | PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR   |
 * | PROFITS; OF BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY  |
 * | OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT         |
 * | (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE    |
 * | USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH     |
 * | DAMAGE.                                                              |
 * +----------------------------------------------------------------------+
 * </pre>
 *
 * @package InterWorx
 * @author  Malin Lorch <mlorch@liquidweb.com>
 * @date    Aug 14, 2020
 */

/**
 *
 * @package Interworx
 * @subpackage Plugin_Acronis
 */
abstract class Model_RecoveryPoint_Abstract extends Model_Acronis
{
    const CENTOS_ROOT = 'centos-root:';

    private $_backupId;

    /**
     * @param string $backupId
     */
    public function __construct($backupId)
    {
        $this->_backupId = $backupId;
    }

    /**
     * Get backup id.
     *
     * @return string
     */
    public function getBackupId()
    {
        return $this->_backupId;
    }

    /**
     * Get subscription id.
     *
     * @return string
     */
    public function getSubscriptionId()
    {
        return $this->_getApi()->getSubscriptionId();
    }

    /**
     * get api with params.
     *
     * @return Model_Api
     */
    protected function _getApi()
    {
        $legacyId = $this->getLegacyId();
        $machineId = $this->getMachineId();
        $Api = parent::_getApi([$legacyId, $machineId]);
        return $Api;
    }


    /**
     * Get legacy id. Expects value to be cached.
     *
     * @return string
     */
    public function getLegacyId()
    {
        if (!isset($this->_legacyId)) {
            $this->_setRecoveryPoint();
        }

        return $this->_legacyId;
    }

    /**
     * Get machine id. Expects value  to be cached.
     *
     * @return string
     */
    public function getMachineId()
    {
        if (!isset($this->_machineId)) {
            $this->_setRecoveryPoint();
        }

        return $this->_machineId;
    }

    /**
     * Get backup creation date.
     *
     * @return string
     */
    public function getCreated()
    {
        if (!isset($this->_created)) {
            $this->_setRecoveryPoint();
        }

        return $this->_created;
    }

    /**
     * Get tenant agent id.
     *
     * @return string
     */
    public function getAgentTenantId()
    {
        if (!isset($this->_tenantAgentId)) {
            $this->_setRecoveryPoint();
        }

        return $this->_tenantAgentId;
    }

    /**
     * Set recovery point.
     */
    private function _setRecoveryPoint()
    {
        $cachedBackup = $this->_getCache()->getBackupById($this->_backupId);
        $this->_throwExceptionIfCacheEmpty($cachedBackup);

        $this->_legacyId = $cachedBackup['legacyId'];
        $this->_machineId = $cachedBackup['machineId'];
        $this->_created = $cachedBackup['created'];
        $this->_tenantAgentId = $cachedBackup['tenantAgentId'];
    }


    /**
     * Throw exception if cache is empty.
     *
     * @param string $cache
     */
    protected function _throwExceptionIfCacheEmpty($cache)
    {
        if (empty($cache)) {
            throw new Exception('Backups cache is empty! Could not find recovery point ids with backup id ' . $this->_backupId);
        }
    }

    /**
     * Get nw root content.
     *
     * @return array
     */
    protected function _getRootContent()
    {
        $Cache = $this->_getCache();
        $centosDirDetails = $Cache->getCentosRoot($this->getBackupId());

        if (empty($centosDirDetails)) {
            $centosDirDetails = $this->_getCentosDirDetailsFromApi();
            $Cache->cacheDir($this->getBackupId(), $centosDirDetails);
        }


        return $centosDirDetails['content'];
    }

    /**
     * get content from /chroot/home/
     *
     * @return array
     */
    protected function _getHomeContent()
    {
        $chrootContent = $this->_getChrootContent();
        $homeId = $this->_getDirIdByName($chrootContent, 'home');
        $homeContent = $this->_getDirectoryContentById($homeId);

        return $homeContent;
    }



    /**
     * Get chroot content.
     *
     * @return array
     */
    private function _getChrootContent()
    {
        $centosContent = $this->_getRootContent();
        $chrootId = $this->_getDirIdByName($centosContent, 'chroot');
        $chrootContent = $this->_getDirectoryContentById($chrootId);
        return $chrootContent;
    }

    /**
     * Get dir id by name.
     *
     * @param array  $dirs
     * @param string $dirName
     */
    protected function _getDirIdByName(array $dirs, $dirName)
    {
        foreach ($dirs as $dir) {
            if ($dir['name'] === $dirName) {
                return $dir['id'];
            }
        }

        throw new Exception_InvalidPath($dirName);
    }

    /**
     * Get directory content.
     * Gets root directory content if no id is specified.
     *
     * @param string $dirId
     */
    protected function _getDirectoryContentById($dirId)
    {
        $Cache = $this->_getCache();
        $dir = $Cache->getDirById($this->getBackupId(), $dirId);

        if (empty($dir)) {
            $dirContent = $this->_getDirContentFromApi($dirId);

            $childPath = $dirContent[0]['path'];
            $name = $this->_getParentName($childPath);
            $path = $this->_getParentPath($childPath);

            $dir = $this->_formatDirDetails($dirId, $dirContent, $name, $path);
            $Cache->cacheDir($this->getBackupId(), $dir);
        }

        return $dir['content'];
    }



    /**
     * Get parent name from child path.
     *
     * @param string $path
     * @return string
     */
    private function _getParentName($path)
    {
        $parentPath = $this->_getParentPath($path);
        $startIndex = strrpos($parentPath, '/');
        $parentName = substr($parentPath, $startIndex);
        return $parentName;
    }

    /**
     * Get parent path from child path.
     *
     * @param string $childPath
     * @return string
     */
    private function _getParentPath($childPath)
    {
        $endIndex = strrpos($childPath, '/');
        $parentPath = substr($childPath, 0, $endIndex);
        return $parentPath;
    }

    /**
     * Get centos dir details.
     *
     * @return array
     */
    private function _getCentosDirDetailsFromApi()
    {
        $root = $this->_getDirContentFromApi();
        $key = array_search('centos-root:', $root, true);
        $centosDir = $root[$key];

        $name = $centosDir['name'];
        $id = $centosDir['id'];
        $path = $centosDir['path'];
        $content = $this->_getDirContentFromApi($id);

        return $this->_formatDirDetails($id, $content, $name, $path);
    }

    /**
     * Format dir details for cache.
     *
     * @param string $dirId
     * @param array  $dirContent
     * @param string $dirName
     * @param string $dirPath
     * @return array
     */
    private function _formatDirDetails($dirId, array $dirContent, $dirName, $dirPath)
    {
        $result = array(
            'id' => $dirId,
            'name' => $dirName,
            'content' => $dirContent,
            'path' => $dirPath
        );

        // save details from parent content, if parent has been cached
        $dirDetails = $this->_getCache()->getFileDetails($this->getBackupId(), $dirId);
        if (isset($dirDetails)) {
            $result = $dirDetails;
            $result['content'] = $dirContent;
        }

        return $result;
    }

    /**
     * Get dir from api.
     *
     * @param string $dirId
     * @return array
     */
    private function _getDirContentFromApi($dirId = null)
    {
        $Api = $this->_getApi();
        $dirContent = $Api->getDirectoryContent($dirId);
        $formatted = $this->_formatDirectories($dirContent);

        return $formatted;
    }



    /**
     * format dirs from api.
     *
     * @param array $apiRes
     */
    protected function _formatDirectories(array $apiRes)
    {
        $filtered = array_filter($apiRes, function ($dir) {
            return $dir['displayName'] !== '';
        });

        $formattedContent = array_map(array($this, '_formatDirContent'), $filtered);
        return $formattedContent;
    }

    /**
     * Format dir content returned by acronis api.
     *
     * @param array $content
     * @return array
     */
    private function _formatDirContent(array $content)
    {
        $displayPath = $this->_excludeCentosRootFromPath($content['fullPath']);
        $date = $this->_formatDate($content['modificationTime']);

        $bytes = $content['size'];
        $size = $this->_convertBytes($bytes);

        return array(
            'displayPath' => $displayPath,
            'path' => $content['fullPath'],
            'id' => $content['id'],
            'name' => $content['displayName'],
            'type' => $content['type'],
            'modified' => $date,
            'size' => $size,
            'parentId' => $content['parentId']
        );
    }

    /**
     * Get substring excluding centos root dir.
     *
     * @param string $path
     * @return string
     */
    private function _excludeCentosRootFromPath($path)
    {
        $pathStart = strpos($path, '/') + 1;
        $centosRootExcluded = substr($path, $pathStart);
        return $centosRootExcluded;
    }

    /**
     * Get db dumps folder id.
     *
     * @return string
     */
    public function getDbDumpsId()
    {
        $homeContent = $this->_getHomeContent();
        $tmpId = $this->_getDirIdByName($homeContent, '.acronis_tmp');
        $tmpContent =  $this->_getDirectoryContentById($tmpId);

        $dbDumpsId = $this->_getDirIdByName($tmpContent, 'pre-acronis-mysql');
        return $dbDumpsId;
    }
}
