<?php

/**
 * InterWorx Hosting Control Panel
 *
 * <pre>
 * +----------------------------------------------------------------------+
 * | Copyright (c) 2000-2010 InterWorx L.L.C., All Rights Reserved.       |
 * +----------------------------------------------------------------------+
 * | Redistribution and use in source form, with or without modification  |
 * | is NOT permitted without consent from the copyright holder.          |
 * |                                                                      |
 * | THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS "AS IS" AND |
 * | ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,    |
 * | THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A          |
 * | PARTICULAR PURPOSE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,    |
 * | EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,  |
 * | PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR   |
 * | PROFITS; OF BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY  |
 * | OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT         |
 * | (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE    |
 * | USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH     |
 * | DAMAGE.                                                              |
 * +----------------------------------------------------------------------+
 * </pre>
 *
 * @package InterWorx
 * @author  Malin Lorch <mlorch@liquidweb.com>
 * @date    Aug 14, 2020
 */

/**
 * @package Interworx
 * @subpackage Plugin_Acronis
 */
abstract class Model_RecoveryPoint_Databases extends Model_RecoveryPoint_Files
{
    const DUMPS_PATH = self::NW_DOWNLOAD_DIR_PATH . '/db-dumps';
    const DUMPS_FOLDERNAME = 'pre-acronis-mysql';
    const DUMP_INFO_USER = 'unixUser';
    const DUMP_INFO_DBS = 'dbNames';


    /**
     * Get dumps info.
     *
     * @return array
     */
    protected function _getDumpsInfoFromApi()
    {
        $dumpsId = $this->getDbDumpsId();
        $content = $this->_getDirectoryContentById($dumpsId);

        $result = array_map(function ($dir) {
            return $this->_getUserDbs($dir['name'], $dir['id']);
        }, $content);

        return array_values(array_filter($result));
    }

    /**
     * Get db info, user & db names.
     *
     * @param string $dirName
     * @param string $dirId
     */
    private function _getUserDbs($dirName, $dirId)
    {
        $isDbDir = strpos($dirName, '-dumps') !== false;

        if ($isDbDir) {
            return array(
                self::DUMP_INFO_USER => str_replace('-dumps', '', $dirName),
                self::DUMP_INFO_DBS => $this->_getDbNamesFromFolderContent($dirId)
            );
        }
    }

    /**
     * exludes mysql db.
     *
     * @param string $dirId
     */
    private function _getDbNamesFromFolderContent($dirId)
    {
        $directories = $this->_getDirectoryContentById($dirId);
        $result = array_map(function ($dir) {
            if ($dir['name'] !== 'mysql') {
                return $dir['name'];
            }
        }, $directories);

        return array_values(array_filter($result));
    }


    /**
     * Run command as root and get output as array.
     *
     * @param string $cmd
     * @return array
     */
    protected function _getCmdOutput($cmd)
    {
        $Command = new Command($cmd);
        $Command->runAsRoot();
        $Command->allowAsterisk();
        $Command->allowAmpersand();
        $Command->allowRedirectOut();
        $Command->exec();
        return $Command->getOutput();
    }

    /**
     * Restore user dumps.
     *
     * @param string $user
     */
    protected function _restoreUserDumps($user)
    {
        $userDumpsId = $this->_getUserDumpsFolderId($user);
        $dirIds = array($userDumpsId);
        $destination = self::DUMPS_PATH;

        $this->_createRestoreAltLocationDir($destination);
        $this->_getApi()->recoverDirectoriesToAltLocation($this->getAgentTenantId(), $dirIds, $destination);
    }

    /**
     * Await restoration. Will wait max 3 minutes for restoration to complete.
     *
     * @param string $unixUser
     */
    protected function _awaitUserDumpsRestoration($unixUser)
    {
        $count = 1;
        while (!$this->_userDumpsRestored($unixUser) && $count < 60) {
            $count++;
            sleep(3);
        }
        if (!$this->_userDumpsRestored($unixUser)) {
            // TODO custom exception? File restoration took to long to proceed?
            throw new Exception('Dumps not restored to tmp location!');
        }
    }

    /**
     * Check if user dumps are restored.
     *
     * @param string $unixUser
     */
    protected function _userDumpsRestored($unixUser)
    {

        $path = self::DUMPS_PATH . "/$unixUser-dumps";
        return $this->_isDir($path);
    }

    /**
     * unzip archives to specified destination.
     *
     * @param string $archive
     * @param string $destination
     */
    protected function _gunzip($archive, $destination)
    {
        $cmd = "gunzip -c $archive > $destination && chown iworx:iworx $destination";
        $Command = new Command($cmd);
        $Command->runAsRoot();
        $Command->allowRedirectOut();
        $Command->allowAmpersand();
        $Command->allowColon();
        $Command->exec();
    }

    /**
     * Get dumps path by unix user.
     *
     * @param string $unixUser
     */
    protected function _getDumpsPathByUnixUser($unixUser)
    {
        $dumpsPath = self::DUMPS_PATH . "/$unixUser-dumps";
        return $dumpsPath;
    }

    /**
     * Get array of archives containing sql file for tables.
     *
     * @param string $parentFolder
     * @return array
     */
    protected function _getTableArchives($parentFolder)
    {
        return $this->_getCmdOutput("ls $parentFolder/*.gz 2> /dev/null");
    }

    /**
     * get sql files in specified location.
     *
     * @param string $sqlPath
     * @param string $dbName
     * @return array
     */
    protected function _getSqlFiles($sqlPath, $dbName)
    {
        return $this->_getCmdOutput("ls $sqlPath/$dbName.*.sql 2> /dev/null");
    }


    /**
     * cleanup dumps.
     */
    protected function _cleanupDumps()
    {
        $dumpsPath = self::DUMPS_PATH;
        $Command = new Command("rm -rf $dumpsPath");
        $Command->runAsRoot();
        $Command->exec();
    }

    /**
     *
     * @param string $user
     * @param array  $dbs
     * @param string $destination
     */
    protected function _restoreUserDbsToAltLocation($user, array $dbs, $destination)
    {
        $this->_createRestoreAltLocationDir($destination);

        $dirIds = $this->_getDbIds($user, $dbs);
        $this->_getApi()->recoverDirectoriesToAltLocation($this->getAgentTenantId(), $dirIds, $destination);
    }

    /**
     * @param string $user
     * @param array  $databases
     */
    private function _getDbIds($user, array $databases)
    {
        $userDumpsContent = $this->_getUserDbFolders($user);

        $ids = array();
        foreach ($userDumpsContent as $dbDumpsFolder) {
            if (in_array($dbDumpsFolder['name'], $databases)) {
                $ids[] = $dbDumpsFolder['id'];
            }
        }

        return $ids;
    }

    /**
     * @param string $user
     * @return array
     */
    private function _getUserDbFolders($user)
    {
        $userDumpsId = $this->_getUserDumpsFolderId($user);
        return $this->_getDirectoryContentById($userDumpsId);
    }

    /**
     * @param string $user
     */
    protected function _getUserDumpsFolderId($user)
    {
        $dumpsId = $this->getDbDumpsId();
        $content = $this->_getDirectoryContentById($dumpsId);
        return $this->_getDirIdByName($content, "$user-dumps");
    }
}
