<?php
/**
 * InterWorx Hosting Control Panel
 *
 * <pre>
 * +----------------------------------------------------------------------+
 * | Copyright (c) 2000-2010 InterWorx L.L.C., All Rights Reserved.       |
 * +----------------------------------------------------------------------+
 * | Redistribution and use in source form, with or without modification  |
 * | is NOT permitted without consent from the copyright holder.          |
 * |                                                                      |
 * | THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS "AS IS" AND |
 * | ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,    |
 * | THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A          |
 * | PARTICULAR PURPOSE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,    |
 * | EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,  |
 * | PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR   |
 * | PROFITS; OF BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY  |
 * | OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT         |
 * | (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE    |
 * | USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH     |
 * | DAMAGE.                                                              |
 * +----------------------------------------------------------------------+
 * </pre>
 *
 * @package InterWorx
 * @author  Malin Lorch <mlorch@liquidweb.com>
 * @date    Aug 21, 2020
 */

/**
 *
 * @package Interworx
 * @subpackage Plugin_Acronis
 */
class Model_RecoveryPoint_Databases_Siteworx extends Model_RecoveryPoint_Databases
{
    private $_unixUser;
    private $_SW;

    /**
     * @param string $backupId
     * @param string $unixUser
     */
    public function __construct($backupId, $unixUser)
    {
        $SW = new SiteWorx();
        $SW->populateUniqname($unixUser);
        if (!$SW->exists()) {
            throw new Exception_AccountDoesNotExist('SW user does not exist! ' . $unixUser);
        }

        $this->_SW = $SW;
        $this->_unixUser = $unixUser;

        parent::__construct($backupId);
    }

    /**
     * Get dbs associated with sw user.
     *
     * @return array
     */
    public function getDbs()
    {
        $Cache = $this->_getCache();
        $dbUsers = $Cache->getDumpsInfo($this->getBackupId());

        if (!isset($dbUsers)) {
            $dbUsers = $this->_getDumpsInfoFromApi();
            $Cache->cacheDumps($this->getBackupId(), $dbUsers);
        }

        foreach ($dbUsers as $dbUser) {
            if ($dbUser[self::DUMP_INFO_USER] === $this->_unixUser) {
                return $dbUser[self::DUMP_INFO_DBS];
            }
        }

        throw new Exception_NoUnixUser($this->_unixUser);
    }

    /**
     * Restore databases.
     *
     * @param array $databases
     */
    public function restoreDatabases(array $databases)
    {
        if (!$this->_userDumpsRestored($this->_unixUser)) {
            $this->_restoreUserDumps($this->_unixUser);
            $this->_awaitUserDumpsRestoration($this->_unixUser);
        }

        $this->_handleRestore($databases);
    }

    /**
     * Resore databases for user.
     *
     * @param array $databases
     */
    private function _handleRestore(array $databases)
    {
        $this->_createSwDumpsDir();
        $this->_placeDumpsInSwUserFolder($databases);
        $this->_restoreDbs($databases);
        $this->_cleanUpSqlFiles();
    }

    /**
     * Create dumps dir.
     */
    private function _createSwDumpsDir()
    {
        $swDumpsPath = $this->_getSwDumpsPath();

        if (!is_dir($swDumpsPath)) {
            mkdir($swDumpsPath, 0755);
        }
    }

    /**
     * Place dumps in sw location.
     *
     * @param array $databases
     */
    private function _placeDumpsInSwUserFolder(array $databases)
    {
        $dumpsPath = $this->_getDumpsPathByUnixUser($this->_unixUser);

        foreach ($databases as $dbName) {
            $archives = $this->_getTableArchives("$dumpsPath/$dbName");
            $this->_unzipFilesInSwLocation($dbName, $archives);
        }
    }

    /**
     * unzip compressed sql files and place in sw location.
     *
     * @param string $dbName
     * @param array  $archives
     */
    private function _unzipFilesInSwLocation($dbName, array $archives)
    {
        foreach ($archives as $archive) {
            $destination = $this->_getSwDestinationPath($dbName, $archive);
            $this->_gunzip($archive, $destination);
        }
    }

    /**
     * Get file name from gz path.
     *
     * @param string $dbName
     * @param string $archive
     * @return string
     */
    private function _getSwDestinationPath($dbName, $archive)
    {
        $uncompressedName = $this->_getUncompressedName($archive);

        $swPath = $this->_getSwDumpsPath() . "/$dbName";
        if (!is_dir($swPath)) {
            mkdir($swPath, 0755);
        }

        return "$swPath/$uncompressedName";
    }

    /**
     * get uncompressed filename.
     *
     * @param string $archive
     */
    private function _getUncompressedName($archive)
    {
        $gzExtensionExcluded = rtrim($archive, '.gz');
        $startPos = strrpos($archive, '/') + 1;
        $uncompressedName = substr($gzExtensionExcluded, $startPos);
        return $uncompressedName;
    }

    /**
     * Get sw dumps path.
     *
     * @return string
     */
    private function _getSwDumpsPath()
    {
        return "/home/$this->_unixUser/var/db-dumps";
    }


    /**
     * Restore sw dbs.
     *
     * @param array $databases
     */
    private function _restoreDbs(array $databases)
    {
        $sqlPath = $this->_getSwDumpsPath();
        foreach ($databases as $dbName) {
            $dbDumpsPath = "$sqlPath/$dbName";
            $sqlFiles = $this->_getSqlFiles($dbDumpsPath, $dbName);
            $this->_restoreDatabase($dbName, $sqlFiles);
        }
    }

    /**
     * Resore database.
     *
     * @param string $database
     * @param array  $sqlFiles
     */
    private function _restoreDatabase($database, array $sqlFiles)
    {
        $Driver = $this->_SW->getMysqlDriver();
        $success = $Driver->importSqlFiles($sqlFiles, $database, $this->_SW);

        if (!$success) {
            throw new Exception('Unable to restore database: ' . $database);
        }
    }

    /**
     * Delete sql files in sw directory.
     */
    private function _cleanUpSqlFiles()
    {
        $path = $this->_getSwDumpsPath();
        $Command = new Command("rm -rf $path/*");
        $Command->allowAsterisk();
        $Command->exec();

        $this->_cleanupDumps();
    }

    /**
     * Resore databases to sw alt location.
     *
     * @param array $dbNames
     */
    public function restoreDatabasesAltLocation(array $dbNames)
    {
        $newDirName = date("Y-m-d-H-i-s");
        $dest = Model_RecoveryPoint_Files_Siteworx::getRestoreAltLocationPath($this->_SW) . "/$newDirName";

        $this->_restoreUserDbsToAltLocation($this->_unixUser, $dbNames, $dest);
    }
}
