<?php
/**
 * InterWorx Hosting Control Panel
 *
 * <pre>
 * +----------------------------------------------------------------------+
 * | Copyright (c) 2000-2010 InterWorx L.L.C., All Rights Reserved.       |
 * +----------------------------------------------------------------------+
 * | Redistribution and use in source form, with or without modification  |
 * | is NOT permitted without consent from the copyright holder.          |
 * |                                                                      |
 * | THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS "AS IS" AND |
 * | ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,    |
 * | THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A          |
 * | PARTICULAR PURPOSE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,    |
 * | EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,  |
 * | PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR   |
 * | PROFITS; OF BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY  |
 * | OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT         |
 * | (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE    |
 * | USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH     |
 * | DAMAGE.                                                              |
 * +----------------------------------------------------------------------+
 * </pre>
 *
 * @package InterWorx
 * @author  Malin Lorch <mlorch@liquidweb.com>
 * @date    Aug 07, 2020
 */

/**
 *
 * @package InterWorx
 * @subpackage Plugin_Acronis
 */
class Model_RecoveryPoint_Databases_UnmappedDbImporter
{
    private $_dsn;
    private $_dbName;
    private $_sqlFiles;
    private $_pass;
    private $_socket;
    private $_username;

    /**
     * construct.
     *
     * @param string $dbName
     * @param array  $sqlFiles
     */
    public function __construct($dbName, array $sqlFiles)
    {
        $this->_dbName = $dbName;
        $this->_sqlFiles = $sqlFiles;
        $this->_dsn = MySQL::getLocalServer()->getDsn();
    }

    /**
     * Recover database.
     */
    public function recoverDatabase()
    {
        $this->_deleteDbIfExists();
        $this->_createDb();
        $this->_populateDb();
    }

    /**
     * Delete db if exists.
     */
    private function _deleteDbIfExists()
    {
        $db = IWorxDbUtil::db_connect($this->_dsn);
        $dbName = $this->_dbName;
        IWorxDbUtil::db_query($db, "DROP DATABASE IF EXISTS $dbName");
    }

    /**
     * Create db.
     */
    private function _createDb()
    {
        $db = IWorxDbUtil::db_connect($this->_dsn);
        $dbName = $this->_dbName;
        IWorxDbUtil::db_query($db, "CREATE DATABASE $dbName");
    }


    /**
     * populate db using sql files.
     */
    private function _populateDb()
    {
        foreach ($this->_sqlFiles as $file) {
            $success = $this->_importSqlFile($file);
            if (!$success) {
                throw new Exception('Unable to inport unmapped database sql file: ' . $file);
            }
        }
    }

    /**
     * import sql file.
     *
     * @param string $file
     */
    private function _importSqlFile($file)
    {
        $pw = $this->_getPass();
        $socket = $this->_getSocket();
        $username = $this->_getUsername();
        $defaults = IWorxDbUtil::getTempDefaultsFile($pw, IWorxDbUtil::USE_IWORX);

        $cmd = Ini::get(Ini::BIN, 'mysql') .
        " --defaults-file={$defaults} " .
        "--user={$username} " .
        "--socket={$socket} " .
        "{$this->_dbName} < {$file}";

        $Command = new Command($cmd);
        $Command->allowRedirectIn();
        $Command->runAsRoot();
        $Command->exec();

        return $Command->wasSuccessful();
    }

    /**
     * Get password.
     *
     * @return string
     */
    private function _getPass()
    {
        if (!isset($this->_pass)) {
            $this->_setDsnDetails();
        }

        return $this->_pass;
    }

    /**
     * Get socket.
     *
     * @return string
     */
    private function _getSocket()
    {
        if (!isset($this->_socket)) {
            $this->_setDsnDetails();
        }

        return $this->_socket;
    }

    /**
     * Get username.
     *
     * @return string
     */
    private function _getUsername()
    {
        if (!isset($this->_username)) {
            $this->_setDsnDetails();
        }

        return $this->_username;
    }

    /**
     * Set dsn details.
     */
    private function _setDsnDetails()
    {
        $parsedDsn = DB::parsedsn($this->_dsn);
        $this->_socket = $parsedDsn['socket'];
        $this->_pass = $parsedDsn['password'];
        $this->_username = $parsedDsn['username'];
    }
}
