<?php

/**
 * InterWorx Hosting Control Panel
 *
 * <pre>
 * +----------------------------------------------------------------------+
 * | Copyright (c) 2000-2010 InterWorx L.L.C., All Rights Reserved.       |
 * +----------------------------------------------------------------------+
 * | Redistribution and use in source form, with or without modification  |
 * | is NOT permitted without consent from the copyright holder.          |
 * |                                                                      |
 * | THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS "AS IS" AND |
 * | ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,    |
 * | THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A          |
 * | PARTICULAR PURPOSE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,    |
 * | EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,  |
 * | PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR   |
 * | PROFITS; OF BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY  |
 * | OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT         |
 * | (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE    |
 * | USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH     |
 * | DAMAGE.                                                              |
 * +----------------------------------------------------------------------+
 * </pre>
 *
 * @package InterWorx
 * @author  Malin Lorch <mlorch@interworx.com>
 * @date    Nov 26, 2019
 */

require_once(__DIR__ . '/../Model/Auth/User.php');
require_once(__DIR__ . '/../Model/Cache.php');
require_once(__DIR__ . '/../Model/ApiClient/OAuth.php');

/**
 * Acronis integration plugin.
 *
 * @package InterWorx
 * @subpackage Plugin_Acronis
 */
class Plugin_Acronis extends Plugin_NodeworxFeature
{
    // TODO place in lang file
    const INVALID_CREDENTIALS = 'Acronis Credentials are invalid.';
    const UNKNOWN_ERROR = 'Something went wrong...';

    const FORM_SECTION_NAME = 'Acronis Account Credentials';
    const INPUT_USERNAME = 'acronis_username';
    const INPUT_PASSWORD = 'acronis_password';
    const INPUT_ACRONIS_URL = 'acronis_url';

    const AB_USERNAME = 'ab_username';
    const AB_PASSWORD = 'ab_password';
    const AB_CREDENTIALS = 'ab_credentials';
    const AB_TENANT_ID = 'ab_tenant_id';
    const AB_USER_ID = 'ab_user_id';
    const AB_URL = 'ab_url';
    const AB_ARRAY_DEVICE_INFO = 'ab_deviceinfo';

    const ACRONIS_ELEMENT_ID = 'acronis';
    const ACRONIS_DASHBOARD_ELEMENT_ID = 'acronis-dashboard';
    const ACRONIS_ELEMENT_CLASS = 'iw-i-plugin';

    private $_cookieAuthApi;
    private $_OAuthApi;

    private $_BackupAgent;
    private $_BackupPlan;
    private $_Backups;
    private $_Cache;
    private $_Auth;
    private $_ActivitiesLogger;
    private $_SwRecoveryPointFiles;
    private $_NwRecoveryPointFiles;
    private $_SwDatabaseRecoveryPoint;
    private $_NwDatabaseRecoveryPoint;

    protected $_nw_user_perm_name = 'acronisbackups';
    protected $_nw_user_perm_label = '##LG_ACRONIS##';
    protected $_reseller_allowed = false;


    /**
     * Decides which category to display plugin under.
     *
     * @return string
     */
    public function getCategory()
    {
        return Plugin_Category::BACKUP_RESTORE;
    }

    /**
     * Initialize edit form.
     *
     * @param Form_NW_Plugins_Edit $Form
     */
    public function initializeMyEditForm(Form_NW_Plugins_Edit $Form)
    {
        if (IW::NW()->getWorkingUser()->isMaster()) {
            $this->_initializeFormMasterUser($Form);
        }
    }

    /**
     * Initialize edit form.
     *
     * @param Form_NW_Plugins_Edit $Form
     */
    private function _initializeFormMasterUser(Form_NW_Plugins_Edit $Form)
    {
        $Group = $Form->addGroup(self::AB_CREDENTIALS, self::FORM_SECTION_NAME);

        $username = new Input_String(self::INPUT_USERNAME);
        $password = new Input_Password(self::INPUT_PASSWORD);
        $url = new Input_String(self::INPUT_ACRONIS_URL);

        $Group->addInput($username)->setLabel('Username')->setRequired(false);
        $Group->addInput($password)->setLabel('Password')->setRequired(false);
        $Group->addInput($url)->setLabel('Url')->setRequired(false);
    }

    /**
     * Validates acronis credentials & installation status before processMyEditForm.
     *
     * @param Form $Form
     * @return boolean
     */
    public function validateMyEditForm(Form $Form)
    {
        if (!IW::NW()->getWorkingUser()->isMaster()) {
            $Form->addError('You do not have permission to active this plugin.');
            return false;
        }

        // only validate if user is enabling plugin
        if ($this->_enableRequest($Form)) {
            try {
                $this->_authorizeUser($Form);
                return true;
            } catch (Exception_InvalidCredentials $e) {
                IW::getLog()->error(self::INVALID_CREDENTIALS);
                $Form->addError(self::INVALID_CREDENTIALS);
                return false;
            } catch (Exception $e) {
                IW::getLog()->error('An error occurred: ' . $e->getMessage() . '. File: ' . $e->getFile() . ', Line: ' . $e->getLine());

                $Form->addError(self::UNKNOWN_ERROR);
                return false;
            }
        }

        return true;
    }

    /**
     * Attempts to authorize user based on form credentials.
     *
     * @param Form $Form
     * @throws Exception
     */
    private function _authorizeUser(Form $Form)
    {
        $username = $Form->getValue(self::INPUT_USERNAME);
        $password = $Form->getValue(self::INPUT_PASSWORD);
        $url = $Form->getValue(self::INPUT_ACRONIS_URL);

        if (empty($username) || empty($password) || empty($url)) {
            throw new Exception('Credentials missing!');
        }

        $User = new Model_Auth_User($username, $password, $url);
        $this->_OAuthApi = new Model_ApiClient_OAuth($User);

        // will throw exception if credentials are wrong
        $this->_OAuthApi->authorizeUser();
    }

    /**
     * Process form.
     *
     * @param Form_NW_Plugins_Edit $Form
     */
    public function processMyEditForm(Form_NW_Plugins_Edit $Form)
    {
        if ($this->_enableRequest($Form)) {
            $password = $Form->getValue(self::INPUT_PASSWORD);

            $encryptedPassword = IWorxEncrypt::encrypt($password);
            $username = $Form->getValue(self::INPUT_USERNAME);
            $tenantId = $this->_OAuthApi->getTenantId();
            $url = $Form->getValue(self::INPUT_ACRONIS_URL);

            $this->setDatastoreVar(self::AB_PASSWORD, $encryptedPassword);
            $this->setDatastoreVar(self::AB_USERNAME, $username);
            $this->setDatastoreVar(self::AB_TENANT_ID, $tenantId);
            $this->setDatastoreVar(self::AB_URL, $url);
        } else {
            $this->getCache()->deleteAll();
        }
    }

    /**
     * Check if user is attempting to disable plugin.
     *
     * @param Form $Form
     * @return boolean
     */
    private function _enableRequest(Form $Form)
    {
        return $Form->getValue('status') === '1';
    }

    /**
     * Add acronis to NW menu when enabled.
     *
     * @param IWorxMenuManager $MenuMan
     */
    public function updateNodeworxMenu(IWorxMenuManager $MenuMan)
    {
        if ($this->isCurrentNwUserAllowed()) {
            $newMenuItemConfig = array(
                'text' => 'Acronis Backup',
                'url' => '/nodeworx/acronis',
                'class' => self::ACRONIS_ELEMENT_CLASS,
                'parent' => 'iw-menu-server',
                'disabled_for_reseller' => '1',
                'perms' => "BRESTORE"
            );

            $addAfterElementId = 'iw-menu-server-ips';
            $newElementId = self::ACRONIS_ELEMENT_ID;

            $MenuMan->addMenuItemAfter($addAfterElementId, $newElementId, $newMenuItemConfig);
        }
    }


    /**
     * Add acronis to NW menu when enabled.
     *
     * @param IWorxMenuManager $MenuMan
     */
    public function updateSiteworxMenu(IWorxMenuManager $MenuMan)
    {
        if (IW::SW()->getWorkingUser()->isMaster()) {
            $newMenuItemConfig = array(
                'text' => 'Acronis Backup',
                'url' => '/siteworx/acronis',
                'class' => self::ACRONIS_ELEMENT_CLASS
            );

            $addAfterElementId = 'iw-menu-file';
            $newElementId = self::ACRONIS_ELEMENT_ID;

            $MenuMan->addMenuItemAfter($addAfterElementId, $newElementId, $newMenuItemConfig);
        }
    }

    /**
     * Return acronis auth api.
     *
     * @return Model_AcronisApi_Interface_Auth
     */
    public function getOAuthApiClient()
    {
        if (!isset($this->_OAuthApi)) {
            $User = $this->getAcronisUser();
            $this->_OAuthApi = new Model_ApiClient_OAuth($User);

            if (!$this->_OAuthApi->isAuthorized()) {
                $this->_OAuthApi->authorizeUser();
            }
        }
        return $this->_OAuthApi;
    }

    /**
     * Get api class for acronis non-public api.
     * Authentication by cookies.
     *
     * @return Model_ApiClient_Client
     */
    public function getCookieAuthApi()
    {
        if (!isset($this->_cookieAuthApi)) {
            $User = $this->getAcronisUser();
            $this->_cookieAuthApi = new Model_ApiClient_CookieAuth($User);
        }
        return $this->_cookieAuthApi;
    }

    /**
     * Get Acronis user.
     *
     * @return Model_Auth_User
     */
    public function getAcronisUser()
    {
        $tenantId = $this->_getTenantId();
        $username = $this->_getUsername();
        $password = $this->_getDecryptedPassword();
        $url = $this->_getUrl();

        $User = new Model_Auth_User($username, $password, $url);
        $User->setTenantId($tenantId);
        return $User;
    }

    /**
     * Check if credentials are saved & validate through acronis api.
     *
     * @return boolean
     */
    public function userIsAuthorized()
    {
        try {
            $OAuthClient = $this->getOAuthApiClient();
            return $OAuthClient->isAuthorized();
        } catch (Exception $e) {
            IW::getLog()->error('Invalid credentials. Message: ' . $e->getMessage() . '. File: ' . $e->getFile() . ', Line: ' . $e->getLine());
            return false;
        }
    }

    /**
     * Get username from datastoreVar.
     *
     * @return string
     */
    private function _getUsername()
    {
        return $this->getDatastoreVar(self::AB_USERNAME);
    }


    /**
     * Get password from datastoreVar.
     *
     * @return string
     */
    private function _getPassword()
    {
        return $this->getDatastoreVar(self::AB_PASSWORD);
    }


    /**
     * Get decrypted password from datastoreVar.
     *
     * @return string
     */
    private function _getDecryptedPassword()
    {
        $psw = $this->_getPassword();
        return IWorxEncrypt::decrypt($psw);
    }


    /**
     * Get url from datastoreVar.
     *
     * @return string
     */
    private function _getUrl()
    {
        return $this->getDatastoreVar(self::AB_URL);
    }

    /**
     * Get tenant id from datastoreVar.
     *
     * @return string
     */
    private function _getTenantId()
    {
        return $this->getDatastoreVar(self::AB_TENANT_ID);
    }

    /**
     * Get backup agent.
     *
     * @return Model_BackupAgent_BackupAgent|Mock
     */
    public function getBackupAgent()
    {
        if (!isset($this->_BackupAgent)) {
            $this->_BackupAgent = new Model_BackupAgent_BackupAgent();
        }

        return $this->_BackupAgent;
    }

    /**
     * Get backup plan.
     *
     * @param string $id
     * @return Model_Backups_BackupPlan|Mock
     */
    public function getBackupPlan($id)
    {
        if (!isset($this->_BackupPlan)) {
            $this->_BackupPlan = new Model_Backups_BackupPlan($id);
        }

        return $this->_BackupPlan;
    }

    /**
     * Get backup plan.
     *
     * @return Model_Backups_Backups|Mock
     */
    public function getBackupsRetriever()
    {
        if (!isset($this->_Backups)) {
            $instanceId = $this->getServer()->getInstanceId();
            $this->_Backups = new Model_Backups_Backups($instanceId);
        }

        return $this->_Backups;
    }

    /**
     * Get files restorer.
     *
     * @param string $backupId
     * @return Model_RecoveryPoint_Files|Mock
     */
    public function getNwFilesRecoveryPoint($backupId)
    {
        if (!isset($this->_NwRecoveryPointFiles)) {
            $this->_NwRecoveryPointFiles = new Model_RecoveryPoint_Files_Nodeworx($backupId);
        }

        return $this->_NwRecoveryPointFiles;
    }

    /**
     * Get sw files restorer.
     *
     * @param string $backupId
     * @return Model_RecoveryPoint_Files|Mock
     */
    public function getSwFilesRecoveryPoint($backupId)
    {
        if (!isset($this->_SwRecoveryPointFiles)) {
            $unixUser = IW::SW()->getUniqname();
            $this->_SwRecoveryPointFiles = new Model_RecoveryPoint_Files_Siteworx($backupId, $unixUser);
        }

        return $this->_SwRecoveryPointFiles;
    }

    /**
     * @param string $backupId
     * @return Model_RecoveryPoint_Databases_Nodeworx|Mock
     */
    public function getNwDatabaseRecoveryPoint($backupId)
    {
        if (!isset($this->_NwDatabaseRecoveryPoint)) {
            $this->_NwDatabaseRecoveryPoint = new Model_RecoveryPoint_Databases_Nodeworx($backupId);
        }

        return $this->_NwDatabaseRecoveryPoint;
    }

    /**
     * @param string $backupId
     * @param string $unixUser
     * @return Model_RecoveryPoint_Databases_Siteworx|Mock
     */
    public function getSwDatabaseRecoveryPoint($backupId, $unixUser)
    {
        if (!isset($this->_SwDatabaseRecoveryPoint)) {
            $this->_SwDatabaseRecoveryPoint = new Model_RecoveryPoint_Databases_Siteworx($backupId, $unixUser);
        }

        return $this->_SwDatabaseRecoveryPoint;
    }

    /**
     * Get cache.
     *
     * @return Model_Cache|Mock
     */
    public function getCache()
    {
        if (!isset($this->_Cache)) {
            $this->_Cache = new Model_Cache();
        }

        return $this->_Cache;
    }

    /**
     * Get plan draft.
     *
     * @return Model_Backups_PlanDraft
     */
    public function getPlanDraft()
    {
        $resourceId = $this->getServer()->getResourceId();
        return new Model_Backups_PlanDraft($resourceId);
    }

    /**
     * Get Server.
     *
     * @return Model_Server_Server|Mock
     */
    public function getServer()
    {
        if (!isset($this->_Server)) {
            $this->_Server = new Model_Server_Server();
        }

        return $this->_Server;
    }

    /**
     * Get auth.
     *
     * @return Model_Auth_Auth|Mock
     */
    public function getAuth()
    {
        if (!isset($this->_Auth)) {
            $this->_Auth = new Model_Auth_Auth();
        }

        return $this->_Auth;
    }

    /**
     * Get auth.
     *
     * @return Model_Alerts_AlertManager|Mock
     */
    public function getAlertManager()
    {
        return new Model_Alerts_AlertManager();
    }

    /**
     * Set mock for testing purposes.
     *
     * @param Mock   $Mock
     * @param string $propertyName
     */
    public function setMock(Mock $Mock, $propertyName)
    {
        $this->$propertyName = $Mock;
    }
}
