<?php
/**
 * InterWorx Hosting Control Panel
 *
 * <pre>
 * +----------------------------------------------------------------------+
 * | Copyright (c) 2000-2010 InterWorx L.L.C., All Rights Reserved.       |
 * +----------------------------------------------------------------------+
 * | Redistribution and use in source form, with or without modification  |
 * | is NOT permitted without consent from the copyright holder.          |
 * |                                                                      |
 * | THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS "AS IS" AND |
 * | ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,    |
 * | THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A          |
 * | PARTICULAR PURPOSE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,    |
 * | EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,  |
 * | PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR   |
 * | PROFITS; OF BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY  |
 * | OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT         |
 * | (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE    |
 * | USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH     |
 * | DAMAGE.                                                              |
 * +----------------------------------------------------------------------+
 * </pre>
 *
 * @package InterWorx
 * @author  Malin Lorch <mlorch@liquidweb.com>
 * @date    Jun 25, 2020
 */

require_once(__DIR__ . '/../mock/OAuthMock.php');
require_once(__DIR__ . '/../mock/BackupAgent.php');
require_once(__DIR__ . '/../mock/BackupPlan.php');
require_once(__DIR__ . '/../mock/Auth.php');
require_once(__DIR__ . '/../mock/RecoveryPoint.php');
require_once(__DIR__ . '/../mock/RecoveryPointSw.php');
require_once(__DIR__ . '/../mock/Server.php');


/**
 *
 * @package Interworx
 * @package Plugin_Acronis
 */
class AcronisCtrlTest extends IWorx_Framework_TestCase
{
    const VIEW_ERROR_MSG = 'ab_error_message';
    const VIEW_INSTALL_AGENT = 'install_backup_agent';
    const EXPECTED_MESSAGE_NOT_AUTHORIZED = '##LG_ACRONIS_NOT_AUTHORIZED##';
    const CTRL_NW = 'Ctrl_Nodeworx_Acronis';
    const CTRL_SW = 'Ctrl_Siteworx_Acronis';

    const API_FAILURE_EXCEPTION = Exception_AcronisApiFailure::class;
    const NO_DB_EXCEPTION = Exception_DbNotFound::class;
    const UNKNOWN_ERROR_EXCEPTION = Exception::class;
    const UNINSTALL_ERROR_EXCEPTION = Exception_BackupAgentUninstallError::class;
    const NO_USER_BACKUP_EXCEPTION = Exception_NoUnixUser::class;
    const INVALID_PATH_EXCEPTION = Exception_InvalidPath::class;

    const INDEX_ACTION = 'index';
    const INSTALL_ACTION = 'installBackupAgentCommit';
    const UNINSTALL_ACTION = 'uninstallBackupAgentCommit';
    const CLOUDSTORAGE_ACTION = 'getStorageSizeCommit';
    const BACKUPPLAN_ACTION = 'getBackupPlanDetailsCommit';
    const BACKUPINFO_ACTION = 'getBackupActivitiesCommit';
    const REDIRECT_ACTION = 'redirectToBackupCloudCommit';
    const REVOKE_PLAN_ACTION = 'revokePlanCommit';
    const ACTIVATE_PLAN_ACTION = 'activatePlanCommit';
    const RUN_BACKUP_ACTION = 'runBackupCommit';
    const LIST_BACKUPS_ACTION = 'listBackupsCommit';
    const GET_ROOT_DIR_ACTION = 'getRootContentCommit';
    const GET_DIR_ACTION = 'getDirectoryContentCommit';
    const RECOVER_DIRS_ACTION = 'recoverDirectoriesCommit';
    const RESTORE_ALT_LOCATION_ACTION = 'restoreToAltLocationCommit';
    const LIST_DBS_ACTION = 'listDatabasesCommit';
    const RESTORE_DBS_ACTION = 'restoreDatabasesCommit';
    const RESTORE_DBS_ALT_LOCATION_ACTION = 'restoreDatabasesAltLocationCommit';

    const THROW_EXCEPTION = true;
    const NO_EXCEPTION = false;
    const SHOULD_FAIL_MSG = 'Should display failure if model throws exception';
    const SHOULD_SUCCEED_MSG = 'Should return success.';

    const LG_UNKNOWN_ERROR = '##LG_UNKNOWN_ERROR##';
    const LG_ACRONIS_API_ERROR = '##LG_ACRONIS_API_FAILURE##';

    /**
     * Make sure oauth mock is used in tests. Defaults to authorized.
     */
    public function setUp()
    {
        $isAuthorized = true;
        $this->_setMockOAuthApi($isAuthorized);
    }



    /**
     * Set moch auth api.
     *
     * @param boolean $isAuthorized
     */
    protected function _setMockOAuthApi($isAuthorized)
    {
        $ApiClient = new OAuthMock($isAuthorized);
        $Plugin = $this->_getPlugin();
        $Plugin->setMock($ApiClient, '_OAuthApi');
    }


    /**
     * Get plugin class.
     *
     * @return Plugin
     */
    private function _getPlugin()
    {
        return IW::PluginManager()->getPlugin('acronis');
    }


    /**
     * send request to ctrl.
     *
     * @param string $action
     * @param array  $input
     * @return Reply
     */
    protected function _sendNwCtrlRequest($action, array $input = array())
    {
        $NW = self::getMasterNodeWorx();
        $Reply = IW::FC()->routeFromPHP(
            self::CTRL_NW,
            $action,
            $input,
            $NW
        );

        return $Reply;
    }

    /**
     * send request to ctrl.
     *
     * @param string $action
     * @param array  $input
     * @return Reply
     */
    protected function _sendSwCtrlRequest($action, array $input = array())
    {
        $SW = $this->getSWAccount();
        $Reply = IW::FC()->routeFromPHP(
            self::CTRL_SW,
            $action,
            $input,
            $SW
        );

        return $Reply;
    }

    /**
     * Set auth mock.
     *
     * @param string $expectedException
     */
    protected function _setAuthMock($expectedException = null)
    {
        $Mock = new Mock_Auth($expectedException);
        $this->_getPlugin()->setMock($Mock, '_Auth');
    }

    /**
     * Set mock server class.
     *
     * @param string $expectedException
     */
    protected function _setMockServer($expectedException = null)
    {
        $MockServer = new Mock_Server($expectedException);
        $this->_getPlugin()->setMock($MockServer, '_Server');
    }

    /**
     * Set mock backup plan.
     *
     * @param string $expectedException
     */
    protected function _setMockBackupPlan($expectedException = null)
    {
        $MockBackupPlan = new Mock_BackupPlan($expectedException);
        $this->_getPlugin()->setMock($MockBackupPlan, '_BackupPlan');
    }

    /**
     * set mock installer.
     *
     * @param boolean $isInstalled       *
     * @param string  $expectedException
     */
    protected function _setMockBackupAgent($isInstalled, $expectedException = null)
    {
        $MockBackupAgent = new Mock_BackupAgent($isInstalled, $expectedException);
        $this->_getPlugin()->setMock($MockBackupAgent, '_BackupAgent');
    }

    /**
     * Set mock recoveryPoint.
     *
     * @param string $expectedException
     */
    protected function _setMockRecoveryPointNwFiles($expectedException = null)
    {
        $MockRecoveryPoint = new Mock_RecoveryPoint($expectedException);
        $this->_getPlugin()->setMock($MockRecoveryPoint, '_NwRecoveryPointFiles');
    }

    /**
     * Set mock recoveryPoint.
     *
     * @param string $expectedException
     */
    protected function _setMockRecoveryPointSwFiles($expectedException = null)
    {
        $MockRecoveryPoint = new Mock_RecoveryPointSw($expectedException);
        $this->_getPlugin()->setMock($MockRecoveryPoint, '_SwRecoveryPointFiles');
    }

    /**
     * Set mock recoveryPoint.
     *
     * @param string $expectedException
     */
    protected function _setMockRecoveryPointNwDatabases($expectedException = null)
    {
        $MockRecoveryPoint = new Mock_RecoveryPoint($expectedException);
        $this->_getPlugin()->setMock($MockRecoveryPoint, '_NwDatabaseRecoveryPoint');
    }

    /**
     * Set mock recoveryPoint.
     *
     * @param string $expectedException
     */
    protected function _setMockRecoveryPointSwDatabases($expectedException = null)
    {
        $MockRecoveryPoint = new Mock_RecoveryPointSw($expectedException);
        $this->_getPlugin()->setMock($MockRecoveryPoint, '_SwDatabaseRecoveryPoint');
    }


    /**
     * @return array
     */
    protected function _getInputDatabases()
    {
        return array(
            'ab_backup_id' => 'backupId',
            'databases' => array('db1'),
            'unixuser' => 'unixuser'
        );
    }


    /**
     * @return array
     */
    protected function _getInputDirIds()
    {
        return array(
            'ab_backup_id' => 'backupId',
            'ab_directories_paths' => array('/path/to/dir')
        );
    }


    /**
     * @return array
     */
    protected function _getInputDirectory()
    {
        return array(
            'ab_directory_path' => '/path',
            'ab_backup_id' => 'backupId'
        );
    }

    /**
     * @return array
     */
    protected function _getInputBackupId()
    {
        return array(
            'ab_backup_id' => 'backupId'
        );
    }

    /**
     * @param Reply $Reply
     */
    protected function _assertUnknownErrorMessage(Reply $Reply)
    {
        $PayloadData = $Reply->getPayloadData();
        $this->assertContains(self::LG_UNKNOWN_ERROR, $PayloadData);
    }

    /**
     * @param Reply $Reply
     */
    protected function _assertApiFailureMessage(Reply $Reply)
    {
        $PayloadData = $Reply->getPayloadData();
        $this->assertContains(self::LG_ACRONIS_API_ERROR, $PayloadData);
    }
}
