<?php

/**
 * InterWorx Hosting Control Panel
 *
 * <pre>
 * +----------------------------------------------------------------------+
 * | Copyright (c) 2000-2010 InterWorx L.L.C., All Rights Reserved.       |
 * +----------------------------------------------------------------------+
 * | Redistribution and use in source form, with or without modification  |
 * | is NOT permitted without consent from the copyright holder.          |
 * |                                                                      |
 * | THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS "AS IS" AND |
 * | ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,    |
 * | THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A          |
 * | PARTICULAR PURPOSE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,    |
 * | EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,  |
 * | PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR   |
 * | PROFITS; OF BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY  |
 * | OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT         |
 * | (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE    |
 * | USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH     |
 * | DAMAGE.                                                              |
 * +----------------------------------------------------------------------+
 * </pre>
 *
 * @package InterWorx
 * @author  Malin Lorch <mlorch@liquidweb.com>
 * @date    Feb 17, 2020
 */

 require_once(__DIR__ . '/AcronisCtrlTest.php');


class Ctrl_Nodeworx_AcronisTest extends AcronisCtrlTest
{
    // const UNIX_USER = 'someuser';
    // const DB_NAME = 'somedbname';
    const BACKUP_ID = 'somebackupid';


    /**
     * index action - should display error message when user not authorized.
     */
    public function testIndexActionShouldFailIfCredentialsInvalid()
    {
        $isAuthorized = false;
        $this->_setMockOAuthApi($isAuthorized);

        $Reply = $this->_sendNwCtrlRequest(self::INDEX_ACTION);
        $this->assertReplyNotSuccessful($Reply, 'Expected index action to return Fail');

        $replyData = $Reply->getData();
        $this->assertContains(self::EXPECTED_MESSAGE_NOT_AUTHORIZED, $replyData);
    }

    /**
     * index action - should not display any error messages if user is authorized.
     */
    public function testIndexActionShouldReturnSuccessfulReplyIfCredentialsValid()
    {
        $Reply = $this->_sendNwCtrlRequest(self::INDEX_ACTION);
        $this->assertReplySuccessful($Reply, 'Expected successful reply from index action when user is authorized.');
    }

    /**
     * Test install backup agent - should succeed if not installed.
     */
    public function testInstallBackupAgentCommitActionShouldDisplaySuccess()
    {
        $isInstalled = false;
        $this->_setMockBackupAgent($isInstalled);

        $Reply = $this->_sendNwCtrlRequest(self::INSTALL_ACTION);
        $this->assertReplySuccessful($Reply);
    }

    /**
     * Test uninstall backup agent - when not installed.
     */
    public function testUninstallBackupAgentCommitActionShouldDisplayUninstallErrorMessage()
    {
        $isInstalled = false;
        $this->_setMockBackupAgent($isInstalled, self::UNINSTALL_ERROR_EXCEPTION);

        $Reply = $this->_sendNwCtrlRequest(self::UNINSTALL_ACTION);
        $this->assertReplyNotSuccessful($Reply, 'Expected unsuccessful reply when backup agent isn\'t installed.');

        $this->_expectUninstallErrorMessage($Reply);
    }

    /**
     * @param Reply $Reply
     */
    private function _expectUninstallErrorMessage(Reply $Reply)
    {
        $PayloadData = $Reply->getPayloadData();
        $this->assertContains('##LG_ACRONIS_UNINSTALL_ERROR##', $PayloadData);
    }


    /**
     * Test uninstall backup agent - when installed.
     */
    public function testUninstallBackupAgentCommitActionShouldDisplaySuccessIfInstalled()
    {
        $isInstalled = true;
        $this->_setMockBackupAgent($isInstalled);

        $Reply = $this->_sendNwCtrlRequest(self::UNINSTALL_ACTION);
        $this->assertReplySuccessful($Reply, 'Expected successful reply when backup agent is installed.');
    }

    /**
     * Test cloud storage action - should catch exception and display 'unknown error'
     */
    public function testCloudstorageCommitActionShouldDisplayUnknownError()
    {
        $this->_setMockServer(self::UNKNOWN_ERROR_EXCEPTION);

        $Reply = $this->_sendNwCtrlRequest(self::CLOUDSTORAGE_ACTION);
        $this->assertReplyNotSuccessful($Reply, self::SHOULD_FAIL_MSG);
        $this->_assertUnknownErrorMessage($Reply);
    }

    /**
     * Test cloud storage action - should catch exception and display 'acronis inaccessible'
     */
    public function testCloudstorageCommitActionShouldDisplayApiFailure()
    {
        $this->_setMockServer(self::API_FAILURE_EXCEPTION);

        $Reply = $this->_sendNwCtrlRequest(self::CLOUDSTORAGE_ACTION);
        $this->assertReplyNotSuccessful($Reply, self::SHOULD_FAIL_MSG);
        $this->_assertApiFailureMessage($Reply);
    }


    /**
     * Test cloud storage action - should return successful reply.
     */
    public function testCloudstorageCommitActionShouldDisplaySuccess()
    {
        $this->_setMockServer();

        $Reply = $this->_sendNwCtrlRequest(self::CLOUDSTORAGE_ACTION);
        $this->assertReplySuccessful($Reply, self::SHOULD_SUCCEED_MSG);
    }

    /**
     * Test BackupPlanStatusCommit action. Should return unsuccessful reply and message 'unknown error'
     */
    public function testBackupPlanStatusCommitActionShouldDisplayUnknownError()
    {
        $this->_setMockBackupPlan(self::UNKNOWN_ERROR_EXCEPTION);
        $this->_setMockServer(self::UNKNOWN_ERROR_EXCEPTION);

        $Reply = $this->_sendNwCtrlRequest(self::BACKUPPLAN_ACTION);
        $this->assertReplyNotSuccessful($Reply, self::SHOULD_FAIL_MSG);
        $this->_assertUnknownErrorMessage($Reply);
    }


    /**
     * Test BackupPlanStatusCommit action. Should return unsuccessful reply and message 'api inaccessible'
     */
    public function testBackupPlanStatusCommitActionShouldDisplayApiError()
    {
        $this->_setMockBackupPlan(self::API_FAILURE_EXCEPTION);
        $this->_setMockServer(self::API_FAILURE_EXCEPTION);

        $Reply = $this->_sendNwCtrlRequest(self::BACKUPPLAN_ACTION);
        $this->assertReplyNotSuccessful($Reply, self::SHOULD_FAIL_MSG);
        $this->_assertApiFailureMessage($Reply);
    }


    /**
     * Test BackupPlanStatusCommit action. Should return successful reply if model throws exception.
     */
    public function testBackupPlanStatusCommitActionShouldSucceed()
    {
        $this->_setMockBackupPlan();
        $this->_setMockServer();


        $Reply = $this->_sendNwCtrlRequest(self::BACKUPPLAN_ACTION);
        $this->assertReplySuccessful($Reply, self::SHOULD_SUCCEED_MSG);
    }

    /**
     * Test backup info commit action. Should return unsuccessful reply and display 'unknown error'
     */
    public function testBackupInfoCommitActionShouldDisplayUnknownError()
    {
        $this->_setMockServer(self::UNKNOWN_ERROR_EXCEPTION);

        $Reply = $this->_sendNwCtrlRequest(self::BACKUPINFO_ACTION);
        $this->assertReplyNotSuccessful($Reply, self::SHOULD_FAIL_MSG);
        $this->_assertUnknownErrorMessage($Reply);
    }
    /**
     * Test backup info commit action. Should return unsuccessful reply and display 'acronis inaccessible'
     */
    public function testBackupInfoCommitActionShouldDisplayApiError()
    {
        $this->_setMockServer(self::API_FAILURE_EXCEPTION);

        $Reply = $this->_sendNwCtrlRequest(self::BACKUPINFO_ACTION);
        $this->assertReplyNotSuccessful($Reply, self::SHOULD_FAIL_MSG);
        $this->_assertApiFailureMessage($Reply);
    }

    /**
     * Test backup info commit action. Should return successful reply.
     */
    public function testBackupInfoCommitActionShouldSucceed()
    {
        $this->_setMockServer();

        $Reply = $this->_sendNwCtrlRequest(self::BACKUPINFO_ACTION);
        $this->assertReplySuccessful($Reply, self::SHOULD_SUCCEED_MSG);
    }

    /**
     * Test redirect to backup cloud. Should fail if model throws exception.
     */
    public function testRedirectToBackupCloudCommitActionShouldDisplayUnknownError()
    {
        $this->_setAuthMock(self::UNKNOWN_ERROR_EXCEPTION);

        $Reply = $this->_sendNwCtrlRequest(self::REDIRECT_ACTION);
        $this->assertReplyNotSuccessful($Reply, self::SHOULD_FAIL_MSG);
        $this->_assertUnknownErrorMessage($Reply);
    }


    /**
     * Test redirect to backup cloud. Should fail if model throws exception.
     */
    public function testRedirectToBackupCloudCommitActionShouldDisplayApiError()
    {
        $this->_setAuthMock(self::API_FAILURE_EXCEPTION);

        $Reply = $this->_sendNwCtrlRequest(self::REDIRECT_ACTION);
        $this->assertReplyNotSuccessful($Reply, self::SHOULD_FAIL_MSG);
        $this->_assertApiFailureMessage($Reply);
    }


    /**
     * Test redirect to backup cloud. Should succeed.
     */
    public function testRedirectToBackupCloudCommitActionShouldSucceed()
    {
        $this->_setAuthMock();

        $Reply = $this->_sendNwCtrlRequest(self::REDIRECT_ACTION);
        $this->assertReplySuccessful($Reply, self::SHOULD_SUCCEED_MSG);
    }


    /**
     * Revoke plan commit action. Should fail if excpetion is thrown in model.
     */
    public function testRevokePlanCommitActionShouldDisplayUnknownError()
    {
        $this->_setMockBackupPlan(self::UNKNOWN_ERROR_EXCEPTION);
        $this->_setMockServer(self::UNKNOWN_ERROR_EXCEPTION);


        $Reply = $this->_sendNwCtrlRequest(self::REVOKE_PLAN_ACTION);
        $this->assertReplyNotSuccessful($Reply, self::SHOULD_FAIL_MSG);
        $this->_assertUnknownErrorMessage($Reply);
    }

    /**
     * Revoke plan commit action. Should fail if excpetion is thrown in model.
     */
    public function testRevokePlanCommitActionShouldDisplayApiFailure()
    {
        $this->_setMockBackupPlan(self::API_FAILURE_EXCEPTION);
        $this->_setMockServer(self::API_FAILURE_EXCEPTION);


        $Reply = $this->_sendNwCtrlRequest(self::REVOKE_PLAN_ACTION);
        $this->assertReplyNotSuccessful($Reply, self::SHOULD_FAIL_MSG);
        $this->_assertApiFailureMessage($Reply);
    }

    /**
     * Revoke plan commit action. Should succeed.
     */
    public function testRevokePlanCommitActionShouldSucceed()
    {
        $this->_setMockBackupPlan();
        $this->_setMockServer();

        $Reply = $this->_sendNwCtrlRequest(self::REVOKE_PLAN_ACTION);
        $this->assertReplySuccessful($Reply, self::SHOULD_SUCCEED_MSG);
    }

    /**
     * Activate plan. Should fail when model throws exception.
     */
    public function testActivatePlanCommitActionShouldDisplayUnknownError()
    {
        $this->_setMockBackupPlan(self::UNKNOWN_ERROR_EXCEPTION);
        $this->_setMockServer(self::UNKNOWN_ERROR_EXCEPTION);

        $Reply = $this->_sendNwCtrlRequest(self::ACTIVATE_PLAN_ACTION);
        $this->assertReplyNotSuccessful($Reply, self::SHOULD_FAIL_MSG);
        $this->_assertUnknownErrorMessage($Reply);
    }

    /**
     * Activate plan. Should succeed.
     */
    public function testActivatePlanCommitActionShouldSucceed()
    {
        $this->_setMockBackupPlan();
        $this->_setMockServer();

        $Reply = $this->_sendNwCtrlRequest(self::ACTIVATE_PLAN_ACTION);
        $this->assertReplySuccessful($Reply, self::SHOULD_SUCCEED_MSG);
    }

    /**
     * Test run backup action. Should display failure if model throws exception.
     */
    public function testRunBackupCommitActionShouldDisplayUnknownError()
    {
        $this->_setMockBackupPlan(self::UNKNOWN_ERROR_EXCEPTION);

        $Reply = $this->_sendNwCtrlRequest(self::RUN_BACKUP_ACTION);
        $this->assertReplyNotSuccessful($Reply, self::SHOULD_FAIL_MSG);
        $this->_assertUnknownErrorMessage($Reply);
    }

    /**
     * Test run backup action. Should display failure if model throws exception.
     */
    public function testRunBackupCommitActionShouldDisplayApiFailure()
    {
        $this->_setMockBackupPlan(self::API_FAILURE_EXCEPTION);

        $Reply = $this->_sendNwCtrlRequest(self::RUN_BACKUP_ACTION);
        $this->assertReplyNotSuccessful($Reply, self::SHOULD_FAIL_MSG);
        $this->_assertApiFailureMessage($Reply);
    }

    /**
     * Test run backup action. Should succeed.
     */
    public function testRunBackupCommitActionShouldSucceed()
    {
        $this->_setMockBackupPlan();
        $this->_setMockServer();

        $Reply = $this->_sendNwCtrlRequest(self::RUN_BACKUP_ACTION);
        $this->assertReplySuccessful($Reply, self::SHOULD_SUCCEED_MSG);
    }

    /**
     * Test listBackupsCommitAction should return failed request if model throws exception.
     */
    public function testListBackupsCommitActionShouldDisplayUnknownError()
    {
        $this->_setMockServer(self::UNKNOWN_ERROR_EXCEPTION);

        $Reply = $this->_sendNwCtrlRequest(self::LIST_BACKUPS_ACTION);
        $this->assertReplyNotSuccessful($Reply, self::SHOULD_FAIL_MSG);
        $this->_assertUnknownErrorMessage($Reply);
    }

    /**
     * Test listBackupsCommitAction should return failed request if model throws exception.
     */
    public function testListBackupsCommitActionShouldDisplayApiFailure()
    {
        $this->_setMockServer(self::API_FAILURE_EXCEPTION);

        $Reply = $this->_sendNwCtrlRequest(self::LIST_BACKUPS_ACTION);
        $this->assertReplyNotSuccessful($Reply, self::SHOULD_FAIL_MSG);
        $this->_assertApiFailureMessage($Reply);
    }


    /**
     * Test listBackupCommitAction should return successful reply when model does not throw any exceptions.
     */
    public function testListBackupsCommitActionShouldSucceed()
    {
        $this->_setMockServer();

        $Reply = $this->_sendNwCtrlRequest(self::LIST_BACKUPS_ACTION);
        $this->assertReplySuccessful($Reply, self::SHOULD_SUCCEED_MSG);
    }

    /**
     * should fail and display 'unknown error'
     */
    public function testGetRootContentCommitActionShouldDisplayUnknownError()
    {
        $this->_setMockRecoveryPointNwFiles(self::UNKNOWN_ERROR_EXCEPTION);

        $input = $this->_getInputBackupId();
        $Reply = $this->_sendNwCtrlRequest(self::GET_ROOT_DIR_ACTION, $input);

        $this->assertReplyNotSuccessful($Reply, self::SHOULD_FAIL_MSG);
        $this->_assertUnknownErrorMessage($Reply);
    }

    /**
     * should fail and display 'api failure'
     */
    public function testGetRootContentCommitActionShouldDisplayApiError()
    {
        $this->_setMockRecoveryPointNwFiles(self::API_FAILURE_EXCEPTION);

        $input = $this->_getInputBackupId();
        $Reply = $this->_sendNwCtrlRequest(self::GET_ROOT_DIR_ACTION, $input);

        $this->assertReplyNotSuccessful($Reply, self::SHOULD_FAIL_MSG);
        $this->_assertApiFailureMessage($Reply);
    }


    /**
     * should succeed
     */
    public function testGetRootContentCommitActionShouldSucceed()
    {
        $this->_setMockRecoveryPointNwFiles();

        $input = $this->_getInputBackupId();
        $Reply = $this->_sendNwCtrlRequest(self::GET_ROOT_DIR_ACTION, $input);

        $this->assertReplySuccessful($Reply, self::SHOULD_SUCCEED_MSG);
    }

    /**
     *
     */
    public function testGetDirectoryContentCommitActionShouldReturnUnknownError()
    {
        $this->_setMockRecoveryPointNwFiles(self::UNKNOWN_ERROR_EXCEPTION);

        $input = $this->_getInputDirectory();
        $Reply = $this->_sendNwCtrlRequest(self::GET_DIR_ACTION, $input);

        $this->assertReplyNotSuccessful($Reply, self::SHOULD_FAIL_MSG);
        $this->_assertUnknownErrorMessage($Reply);
    }

    /**
     *
     */
    public function testGetDirectoryContentCommitActionShouldReturnApiFailure()
    {
        $this->_setMockRecoveryPointNwFiles(self::API_FAILURE_EXCEPTION);

        $input = $this->_getInputDirectory();
        $Reply = $this->_sendNwCtrlRequest(self::GET_DIR_ACTION, $input);

        $this->assertReplyNotSuccessful($Reply, self::SHOULD_FAIL_MSG);
        $this->_assertApiFailureMessage($Reply);
    }

    /**
     * make sure ctrl request fails if model throws exception.
     */
    public function testRecoverDirectoriesCommitActionShouldDisplayUnknownError()
    {
        $this->_setMockRecoveryPointNwFiles(self::UNKNOWN_ERROR_EXCEPTION);
        $input = $this->_getInputDirIds();
        $Reply = $this->_sendNwCtrlRequest(self::RECOVER_DIRS_ACTION, $input);
        $this->assertReplyNotSuccessful($Reply, self::SHOULD_FAIL_MSG);
        $this->_assertUnknownErrorMessage($Reply);
    }

    /**
     * make sure ctrl request fails if model throws exception.
     */
    public function testRecoverDirectoriesCommitActionShouldDisplayApiFailure()
    {
        $this->_setMockRecoveryPointNwFiles(self::API_FAILURE_EXCEPTION);
        $input = $this->_getInputDirIds();
        $Reply = $this->_sendNwCtrlRequest(self::RECOVER_DIRS_ACTION, $input);
        $this->assertReplyNotSuccessful($Reply, self::SHOULD_FAIL_MSG);
        $this->_assertApiFailureMessage($Reply);
    }

    /**
     * make sure ctrl request fails if model throws exception.
     */
    public function testRecoverDirectoriesCommitActionShouldDisplayApiFailed()
    {
        $this->_setMockRecoveryPointNwFiles(self::API_FAILURE_EXCEPTION);
        $input = $this->_getInputDirIds();
        $Reply = $this->_sendNwCtrlRequest(self::RECOVER_DIRS_ACTION, $input);

        $this->assertReplyNotSuccessful($Reply, self::SHOULD_FAIL_MSG);
        $this->_assertApiFailureMessage($Reply);
    }

    /**
     * Test recoverDirectoriesCommitAction should succeed.
     */
    public function testRecoverDirectoriesCommitActionShouldSucceed()
    {
        $this->_setMockRecoveryPointNwFiles();

        $input = $this->_getInputDirIds();
        $Reply = $this->_sendNwCtrlRequest(self::RECOVER_DIRS_ACTION, $input);
        $this->assertReplySuccessful($Reply, self::SHOULD_SUCCEED_MSG);
    }

    /**
     * Test restoreToAltLocationCommitAction should succeed.
     */
    public function testRestoreToAltLocationCommitActionShouldDisplayUnknownError()
    {
        $this->_setMockRecoveryPointNwFiles(self::UNKNOWN_ERROR_EXCEPTION);
        $input = $this->_getInputDirIds();
        $Reply = $this->_sendNwCtrlRequest(self::RESTORE_ALT_LOCATION_ACTION, $input);

        $this->assertReplyNotSuccessful($Reply, self::SHOULD_FAIL_MSG);
        $this->_assertUnknownErrorMessage($Reply);
    }

    /**
     * Test restoreToAltLocationCommitAction should succeed.
     */
    public function testRestoreToAltLocationCommitActionShouldDisplayApiFailure()
    {
        $this->_setMockRecoveryPointNwFiles(self::API_FAILURE_EXCEPTION);
        $input = $this->_getInputDirIds();
        $Reply = $this->_sendNwCtrlRequest(self::RESTORE_ALT_LOCATION_ACTION, $input);

        $this->assertReplyNotSuccessful($Reply, self::SHOULD_FAIL_MSG);
        $this->_assertApiFailureMessage($Reply);
    }

    /**
     * Test restoreToAltLocationCommitAction should succeed.
     */
    public function testRestoreToAltLocationCommitActionShouldSucceed()
    {
        $this->_setMockRecoveryPointNwFiles();
        $input = $this->_getInputDirIds();
        $Reply = $this->_sendNwCtrlRequest(self::RESTORE_ALT_LOCATION_ACTION, $input);

        $this->assertReplySuccessful($Reply, self::SHOULD_SUCCEED_MSG);
    }

    /**
     * Should fail and display 'unknown error'.
     */
    public function testListDatabasesCommitActionShouldDisplayUnknownError()
    {
        $this->_setMockRecoveryPointNwDatabases(self::UNKNOWN_ERROR_EXCEPTION);

        $input = $this->_getInputBackupId();
        $Reply = $this->_sendNwCtrlRequest(self::LIST_DBS_ACTION, $input);
        $this->assertReplyNotSuccessful($Reply, self::SHOULD_FAIL_MSG);
        $this->_assertUnknownErrorMessage($Reply);
    }

    /**
     * Should fail and display 'api failure'.
     */
    public function testListDatabasesCommitActionShouldDisplayApiFailure()
    {
        $this->_setMockRecoveryPointNwDatabases(self::API_FAILURE_EXCEPTION);

        $input = $this->_getInputBackupId();
        $Reply = $this->_sendNwCtrlRequest(self::LIST_DBS_ACTION, $input);
        $this->assertReplyNotSuccessful($Reply, self::SHOULD_FAIL_MSG);
        $this->_assertApiFailureMessage($Reply);
    }

    /**
     * Should succeed when model actions are successful.
     */
    public function testListDatabasesCommitActionShouldSucceed()
    {
        $this->_setMockRecoveryPointNwDatabases();

        $input = $this->_getInputBackupId();
        $Reply = $this->_sendNwCtrlRequest(self::LIST_DBS_ACTION, $input);
        $this->assertReplySuccessful($Reply, self::SHOULD_SUCCEED_MSG);
    }

    /**
     * Should fail and display 'unknown error'.
     */
    public function testRecoverDatabasesCommitActionShouldDisplayUnknownError()
    {
        $this->_setMockRecoveryPointNwDatabases(self::UNKNOWN_ERROR_EXCEPTION);

        $input = $this->_getInputDatabases();
        $Reply = $this->_sendNwCtrlRequest(self::RESTORE_DBS_ACTION, $input);

        $this->assertReplyNotSuccessful($Reply, self::SHOULD_FAIL_MSG);
        $this->_assertUnknownErrorMessage($Reply);
    }

    /**
     * Should fail and display 'api failure'.
     */
    public function testRecoverDatabasesCommitActionShouldDisplayApiFailure()
    {
        $this->_setMockRecoveryPointNwDatabases(self::API_FAILURE_EXCEPTION);

        $input = $this->_getInputDatabases();
        $Reply = $this->_sendNwCtrlRequest(self::RESTORE_DBS_ACTION, $input);

        $this->assertReplyNotSuccessful($Reply, self::SHOULD_FAIL_MSG);
        $this->_assertApiFailureMessage($Reply);
    }

    /**
     * Should succeed.
     */
    public function testRecoverDatabasesCommitActionShouldSucceed()
    {
        $this->_setMockRecoveryPointNwDatabases();

        $input = $this->_getInputDatabases();
        $Reply = $this->_sendNwCtrlRequest(self::RESTORE_DBS_ACTION, $input);

        $this->assertReplySuccessful($Reply, self::SHOULD_SUCCEED_MSG);
    }

    /**
     * Should fail and display 'unknown error'.
     */
    public function testRecoverDatabasesAltLocationCommitActionShouldDisplayUnknownError()
    {
        $this->_setMockRecoveryPointNwDatabases(self::UNKNOWN_ERROR_EXCEPTION);

        $input = $this->_getInputDatabases();
        $Reply = $this->_sendNwCtrlRequest(self::RESTORE_DBS_ALT_LOCATION_ACTION, $input);

        $this->assertReplyNotSuccessful($Reply, self::SHOULD_FAIL_MSG);
        $this->_assertUnknownErrorMessage($Reply);
    }

    /**
     * Should fail and display 'api failure'.
     */
    public function testRecoverDatabasesAltLocationCommitActionShouldDisplayApiFailure()
    {
        $this->_setMockRecoveryPointNwDatabases(self::API_FAILURE_EXCEPTION);

        $input = $this->_getInputDatabases();
        $Reply = $this->_sendNwCtrlRequest(self::RESTORE_DBS_ALT_LOCATION_ACTION, $input);

        $this->assertReplyNotSuccessful($Reply, self::SHOULD_FAIL_MSG);
        $this->_assertApiFailureMessage($Reply);
    }
}
