<?php

/**
 * InterWorx Hosting Control Panel
 *
 * <pre>
 * +----------------------------------------------------------------------+
 * | Copyright (c) 2000-2010 InterWorx L.L.C., All Rights Reserved.       |
 * +----------------------------------------------------------------------+
 * | Redistribution and use in source form, with or without modification  |
 * | is NOT permitted without consent from the copyright holder.          |
 * |                                                                      |
 * | THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS "AS IS" AND |
 * | ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,    |
 * | THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A          |
 * | PARTICULAR PURPOSE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,    |
 * | EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,  |
 * | PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR   |
 * | PROFITS; OF BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY  |
 * | OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT         |
 * | (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE    |
 * | USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH     |
 * | DAMAGE.                                                              |
 * +----------------------------------------------------------------------+
 * </pre>
 *
 * @package InterWorx
 * @author  Malin Lorch <mlorch@liquidweb.com>
 * @date    Jun 25, 2020
 */

require_once(__DIR__ . '/AcronisCtrlTest.php');

class Ctrl_Siteworx_AcronisTest extends AcronisCtrlTest
{
    const LG_NO_BACKUP = '##LG_ACRONIS_NO_BACKUP##';
    const LG_INVALID_PATH = '##LG_ACRONIS_INVALID_PATH##';
    const LG_NO_DB = '##LG_ACRONIS_NO_DB##';

    /**
     * index action - should display error message when user not authorized.
     */
    public function testIndexActionShouldFail()
    {
        $isAuthorized = false;
        $this->_setMockOAuthApi($isAuthorized);

        $Reply = $this->_sendSwCtrlRequest(self::INDEX_ACTION);
        $this->assertReplyNotSuccessful($Reply, 'Expected index action to fail when user not authorized');

        $replyData = $Reply->getData();
        $this->assertContains(self::EXPECTED_MESSAGE_NOT_AUTHORIZED, $replyData);
    }

    /**
     * index action
     */
    public function testIndexActionShouldSucceed()
    {
        $Reply = $this->_sendSwCtrlRequest(self::INDEX_ACTION);
        $this->assertReplySuccessful($Reply, 'Expected successful reply');
    }


    /**
     * Test listBackupsCommitAction should return failed request if model throws exception.
     */
    public function testListBackupsCommitActionShouldDisplayUnknownError()
    {
        $this->_setMockServer(self::UNKNOWN_ERROR_EXCEPTION);

        $Reply = $this->_sendSwCtrlRequest(self::LIST_BACKUPS_ACTION);
        $this->assertReplyNotSuccessful($Reply, self::SHOULD_FAIL_MSG);
        $this->_assertUnknownErrorMessage($Reply);
    }


    /**
     * Test listBackupsCommitAction should return failed request if model throws exception.
     */
    public function testListBackupsCommitActionShouldDisplayApiFailure()
    {
        $this->_setMockServer(self::API_FAILURE_EXCEPTION);

        $Reply = $this->_sendSwCtrlRequest(self::LIST_BACKUPS_ACTION);
        $this->assertReplyNotSuccessful($Reply, self::SHOULD_FAIL_MSG);
        $this->_assertApiFailureMessage($Reply);
    }


    /**
     * Test listBackupCommitAction should return successful reply when model does not throw any exceptions.
     */
    public function testListBackupsCommitActionShouldSucceed()
    {
        $this->_setMockServer();

        $Reply = $this->_sendSwCtrlRequest(self::LIST_BACKUPS_ACTION);
        $this->assertReplySuccessful($Reply, self::SHOULD_SUCCEED_MSG);
    }

    /**
     * should fail and display 'unknown error'
     */
    public function testGetRootContentCommitActionShouldDisplayUnknownError()
    {
        $this->_setMockRecoveryPointSwFiles(self::UNKNOWN_ERROR_EXCEPTION);

        $input = $this->_getInputBackupId();
        $Reply = $this->_sendSwCtrlRequest(self::GET_ROOT_DIR_ACTION, $input);

        $this->assertReplyNotSuccessful($Reply, self::SHOULD_FAIL_MSG);
        $this->_assertUnknownErrorMessage($Reply);
    }

    /**
     * should fail and display 'api failure'
     */
    public function testGetRootContentCommitActionShouldDisplayApiError()
    {
        $this->_setMockRecoveryPointSwFiles(self::API_FAILURE_EXCEPTION);

        $input = $this->_getInputBackupId();
        $Reply = $this->_sendSwCtrlRequest(self::GET_ROOT_DIR_ACTION, $input);

        $this->assertReplyNotSuccessful($Reply, self::SHOULD_FAIL_MSG);
        $this->_assertApiFailureMessage($Reply);
    }

    /**
     * should fail and display 'no backup'
     */
    public function testGetRootContentCommitActionShouldDisplayNoBackup()
    {
        $this->_setMockRecoveryPointSwFiles(self::NO_USER_BACKUP_EXCEPTION);

        $input = $this->_getInputBackupId();
        $Reply = $this->_sendSwCtrlRequest(self::GET_ROOT_DIR_ACTION, $input);

        $this->assertReplyNotSuccessful($Reply, self::SHOULD_FAIL_MSG);
        $this->_assertNoBackupMessage($Reply);
    }

    /**
     * @param Reply $Reply
     */
    private function _assertNoBackupMessage(Reply $Reply)
    {
        $PayloadData = $Reply->getPayloadData();
        $this->assertContains(self::LG_NO_BACKUP, $PayloadData);
    }


    /**
     * should succeed
     */
    public function testGetRootContentCommitActionShouldSucceed()
    {
        $this->_setMockRecoveryPointSwFiles();

        $input = $this->_getInputBackupId();
        $Reply = $this->_sendSwCtrlRequest(self::GET_ROOT_DIR_ACTION, $input);

        $this->assertReplySuccessful($Reply, self::SHOULD_SUCCEED_MSG);
    }



    /**
     *
     */
    public function testGetDirectoryContentCommitActionShouldReturnUnknownError()
    {
        $this->_setMockRecoveryPointSwFiles(self::UNKNOWN_ERROR_EXCEPTION);

        $input = $this->_getInputDirectory();
        $Reply = $this->_sendSwCtrlRequest(self::GET_DIR_ACTION, $input);

        $this->assertReplyNotSuccessful($Reply, self::SHOULD_FAIL_MSG);
        $this->_assertUnknownErrorMessage($Reply);
    }

    /**
     *
     */
    public function testGetDirectoryContentCommitActionShouldReturnApiFailure()
    {
        $this->_setMockRecoveryPointSwFiles(self::INVALID_PATH_EXCEPTION);

        $input = $this->_getInputDirectory();
        $Reply = $this->_sendSwCtrlRequest(self::GET_DIR_ACTION, $input);

        $this->assertReplyNotSuccessful($Reply, self::SHOULD_FAIL_MSG);
        $this->_assertPathInvalidMessage($Reply);
    }

    /**
     * @param Reply $Reply
     */
    private function _assertPathInvalidMessage(Reply $Reply)
    {
        $PayloadData = $Reply->getPayloadData();
        $this->assertContains(self::LG_INVALID_PATH, $PayloadData);
    }

    /**
     * make sure ctrl request fails if model throws exception.
     */
    public function testRecoverDirectoriesCommitActionShouldDisplayUnknownError()
    {
        $this->_setMockRecoveryPointSwFiles(self::UNKNOWN_ERROR_EXCEPTION);
        $input = $this->_getInputDirIds();
        $Reply = $this->_sendSwCtrlRequest(self::RECOVER_DIRS_ACTION, $input);
        $this->assertReplyNotSuccessful($Reply, self::SHOULD_FAIL_MSG);
        $this->_assertUnknownErrorMessage($Reply);
    }

    /**
     *
     */
    public function testRecoverDirectoriesCommitActionShouldReturnApiFailure()
    {
        $this->_setMockRecoveryPointSwFiles(self::INVALID_PATH_EXCEPTION);

        $input = $this->_getInputDirIds();
        $Reply = $this->_sendSwCtrlRequest(self::RECOVER_DIRS_ACTION, $input);

        $this->assertReplyNotSuccessful($Reply, self::SHOULD_FAIL_MSG);
        $this->_assertPathInvalidMessage($Reply);
    }



    /**
     * make sure ctrl request fails if model throws exception.
     */
    public function testRecoverDirectoriesCommitActionShouldDisplayApiFailed()
    {
        $this->_setMockRecoveryPointSwFiles(self::API_FAILURE_EXCEPTION);
        $input = $this->_getInputDirIds();
        $Reply = $this->_sendSwCtrlRequest(self::RECOVER_DIRS_ACTION, $input);

        $this->assertReplyNotSuccessful($Reply, self::SHOULD_FAIL_MSG);
        $this->_assertApiFailureMessage($Reply);
    }

    /**
     * Test recoverDirectoriesCommitAction should succeed.
     */
    public function testRecoverDirectoriesCommitActionShouldSucceed()
    {
        $this->_setMockRecoveryPointSwFiles();

        $input = $this->_getInputDirIds();
        $Reply = $this->_sendSwCtrlRequest(self::RECOVER_DIRS_ACTION, $input);
        $this->assertReplySuccessful($Reply, self::SHOULD_SUCCEED_MSG);
    }

    /**
     * Test restoreToAltLocationCommitAction should succeed.
     */
    public function testRestoreToAltLocationCommitActionShouldDisplayUnknownError()
    {
        $this->_setMockRecoveryPointSwFiles(self::UNKNOWN_ERROR_EXCEPTION);
        $input = $this->_getInputDirIds();
        $Reply = $this->_sendSwCtrlRequest(self::RESTORE_ALT_LOCATION_ACTION, $input);

        $this->assertReplyNotSuccessful($Reply, self::SHOULD_FAIL_MSG);
        $this->_assertUnknownErrorMessage($Reply);
    }

    /**
     * Test restoreToAltLocationCommitAction should succeed.
     */
    public function testRestoreToAltLocationCommitActionShouldDisplayApiFailure()
    {
        $this->_setMockRecoveryPointSwFiles(self::API_FAILURE_EXCEPTION);
        $input = $this->_getInputDirIds();
        $Reply = $this->_sendSwCtrlRequest(self::RESTORE_ALT_LOCATION_ACTION, $input);

        $this->assertReplyNotSuccessful($Reply, self::SHOULD_FAIL_MSG);
        $this->_assertApiFailureMessage($Reply);
    }

    /**
     * Test restoreToAltLocationCommitAction should succeed.
     */
    public function testRestoreToAltLocationCommitActionShouldDisplayInvalidPath()
    {
        $this->_setMockRecoveryPointSwFiles(self::INVALID_PATH_EXCEPTION);
        $input = $this->_getInputDirIds();
        $Reply = $this->_sendSwCtrlRequest(self::RESTORE_ALT_LOCATION_ACTION, $input);

        $this->assertReplyNotSuccessful($Reply, self::SHOULD_FAIL_MSG);
        $this->_assertPathInvalidMessage($Reply);
    }

    /**
     * Test restoreToAltLocationCommitAction should succeed.
     */
    public function testRestoreToAltLocationCommitActionShouldSucceed()
    {
        $this->_setMockRecoveryPointSwFiles();
        $input = $this->_getInputDirIds();
        $Reply = $this->_sendSwCtrlRequest(self::RESTORE_ALT_LOCATION_ACTION, $input);

        $this->assertReplySuccessful($Reply, self::SHOULD_SUCCEED_MSG);
    }

    /**
     * Should fail and display 'unknown error'.
     */
    public function testListDatabasesCommitActionShouldDisplayUnknownError()
    {
        $this->_setMockRecoveryPointSwDatabases(self::UNKNOWN_ERROR_EXCEPTION);

        $input = $this->_getInputBackupId();
        $Reply = $this->_sendSwCtrlRequest(self::LIST_DBS_ACTION, $input);
        $this->assertReplyNotSuccessful($Reply, self::SHOULD_FAIL_MSG);
        $this->_assertUnknownErrorMessage($Reply);
    }

    /**
     * Should fail and display 'api failure'.
     */
    public function testListDatabasesCommitActionShouldDisplayApiFailure()
    {
        $this->_setMockRecoveryPointSwDatabases(self::API_FAILURE_EXCEPTION);

        $input = $this->_getInputBackupId();
        $Reply = $this->_sendSwCtrlRequest(self::LIST_DBS_ACTION, $input);

        $this->assertReplyNotSuccessful($Reply, self::SHOULD_FAIL_MSG);
        $this->_assertApiFailureMessage($Reply);
    }

    /**
     * Should succeed when model actions are successful.
     */
    public function testListDatabasesCommitActionShouldSucceed()
    {
        $this->_setMockRecoveryPointSwDatabases();

        $input = $this->_getInputBackupId();
        $Reply = $this->_sendSwCtrlRequest(self::LIST_DBS_ACTION, $input);
        $this->assertReplySuccessful($Reply, self::SHOULD_SUCCEED_MSG);
    }

    /**
     * Should fail and display 'unknown error'.
     */
    public function testRecoverDatabasesCommitActionShouldDisplayUnknownError()
    {
        $this->_setMockRecoveryPointSwDatabases(self::UNKNOWN_ERROR_EXCEPTION);

        $input = $this->_getInputDatabases();
        $Reply = $this->_sendSwCtrlRequest(self::RESTORE_DBS_ACTION, $input);

        $this->assertReplyNotSuccessful($Reply, self::SHOULD_FAIL_MSG);
        $this->_assertUnknownErrorMessage($Reply);
    }

    /**
     * Should fail and display 'api failure'.
     */
    public function testRecoverDatabasesCommitActionShouldDisplayApiFailure()
    {
        $this->_setMockRecoveryPointSwDatabases(self::API_FAILURE_EXCEPTION);

        $input = $this->_getInputDatabases();
        $Reply = $this->_sendSwCtrlRequest(self::RESTORE_DBS_ACTION, $input);

        $this->assertReplyNotSuccessful($Reply, self::SHOULD_FAIL_MSG);
        $this->_assertApiFailureMessage($Reply);
    }

    /**
     * Should fail and display 'no such db'.
     */
    public function testRecoverDatabasesCommitActionShouldDisplayNoDb()
    {
        $this->_setMockRecoveryPointSwDatabases(self::NO_DB_EXCEPTION);

        $input = $this->_getInputDatabases();
        $Reply = $this->_sendSwCtrlRequest(self::RESTORE_DBS_ACTION, $input);

        $this->assertReplyNotSuccessful($Reply, self::SHOULD_FAIL_MSG);
        $this->_assertNoDbFailure($Reply);
    }

    /**
     * @param Reply $Reply
     */
    private function _assertNoDbFailure(Reply $Reply)
    {
        $PayloadData = $Reply->getPayloadData();
        $this->assertContains(self::LG_NO_DB, $PayloadData);
    }

    /**
     * Should succeed.
     */
    public function testRecoverDatabasesCommitActionShouldSucceed()
    {
        $this->_setMockRecoveryPointSwDatabases();

        $input = $this->_getInputDatabases();
        $Reply = $this->_sendSwCtrlRequest(self::RESTORE_DBS_ACTION, $input);

        $this->assertReplySuccessful($Reply, self::SHOULD_SUCCEED_MSG);
    }

    /**
     * Should fail and display 'unknown error'.
     */
    public function testRecoverDatabasesAltLocationCommitActionShouldDisplayUnknownError()
    {
        $this->_setMockRecoveryPointSwDatabases(self::UNKNOWN_ERROR_EXCEPTION);

        $input = $this->_getInputDatabases();
        $Reply = $this->_sendSwCtrlRequest(self::RESTORE_DBS_ALT_LOCATION_ACTION, $input);

        $this->assertReplyNotSuccessful($Reply, self::SHOULD_FAIL_MSG);
        $this->_assertUnknownErrorMessage($Reply);
    }

    /**
     * Should fail and display 'api failure'.
     */
    public function testRecoverDatabasesAltLocationCommitActionShouldDisplayApiFailure()
    {
        $this->_setMockRecoveryPointSwDatabases(self::API_FAILURE_EXCEPTION);

        $input = $this->_getInputDatabases();
        $Reply = $this->_sendSwCtrlRequest(self::RESTORE_DBS_ALT_LOCATION_ACTION, $input);

        $this->assertReplyNotSuccessful($Reply, self::SHOULD_FAIL_MSG);
        $this->_assertApiFailureMessage($Reply);
    }

    /**
     * Should fail and display 'no such db'.
     */
    public function testRecoverDatabasesAltLocationCommitActionShouldDisplayNoDb()
    {
        $this->_setMockRecoveryPointSwDatabases(self::NO_DB_EXCEPTION);

        $input = $this->_getInputDatabases();
        $Reply = $this->_sendSwCtrlRequest(self::RESTORE_DBS_ALT_LOCATION_ACTION, $input);

        $this->assertReplyNotSuccessful($Reply, self::SHOULD_FAIL_MSG);
        $this->_assertNoDbFailure($Reply);
    }

    /**
     * Should succeed
     */
    public function testRecoverDatabasesAltLocationCommitActionShouldSucceed()
    {
        $this->_setMockRecoveryPointSwDatabases();

        $input = $this->_getInputDatabases();
        $Reply = $this->_sendSwCtrlRequest(self::RESTORE_DBS_ALT_LOCATION_ACTION, $input);

        $this->assertReplySuccessful($Reply, self::SHOULD_SUCCEED_MSG);
    }
}
