<?php

/**
 * InterWorx Hosting Control Panel
 *
 * <pre>
 * +----------------------------------------------------------------------+
 * | Copyright (c) 2000-2010 InterWorx L.L.C., All Rights Reserved.       |
 * +----------------------------------------------------------------------+
 * | Redistribution and use in source form, with or without modification  |
 * | is NOT permitted without consent from the copyright holder.          |
 * |                                                                      |
 * | THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS "AS IS" AND |
 * | ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,    |
 * | THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A          |
 * | PARTICULAR PURPOSE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,    |
 * | EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,  |
 * | PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR   |
 * | PROFITS; OF BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY  |
 * | OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT         |
 * | (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE    |
 * | USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH     |
 * | DAMAGE.                                                              |
 * +----------------------------------------------------------------------+
 * </pre>
 *
 * @package InterWorx
 * @author  Malin Lorch <mlorch@liquidweb.com>
 * @date    Apr 27, 2020
 */

require_once(__DIR__ . '/Acronis.php');
require_once(__DIR__ . '/../../Model/Backups/BackupPlan.php');
require_once(__DIR__ . '/../../Model/Backups/Api.php');

/**
 *
 *
 * @package Interworx
 * @subpackage Plugin_Acronis
 */
class BackupPlanTest extends AcronisTest
{
    const PLAN_ID = 'planid';
    const PLAN_DAYS = ['monday', 'tuesday'];
    const TYPE = 'daily';
    const ENABLED = true;

    private $_SUT;
    private $_apiMock;
    private $_date;
    private $_isEnabled;
    private $_prePostCommands;

    /**
     * Set up sut & mock before each test.
     */
    public function setUp()
    {
        $this->_apiMock = $this->createMock(Model_Backups_Api::class);
        $this->_SUT = $this->_getSutPartialMock(Model_Backups_BackupPlan::class, $this->_apiMock, self::PLAN_ID);
        $this->_date = date(DATE_RFC2822);
    }

    /**
     * Expect exception if api does not return a plan.
     */
    public function testGetPlanDetailsShouldThrowExceptionIfPlanDoesNotExist()
    {
        $this->_expectException();
        $this->_SUT->getPlanDetails();
    }

    /**
     * Get plan details should return formatted api response.
     */
    public function testGetPlanDetailsShouldReturnPlan()
    {
        $plan = $this->_getPlan();
        $this->_configureApiGetPlanByIdReturnValue($plan);

        $actual = $this->_SUT->getPlanDetails();
        $this->_assertKeysPlanDetails($actual);
        $this->_assertPlanValues($actual);
    }

    /**
     * Assert plan details has keys.
     *
     * @param array $actual
     */
    private function _assertKeysPlanDetails(array $actual)
    {
        $this->assertArrayHasKey('id', $actual);
        $this->assertArrayHasKey('enabled', $actual);
        $this->assertArrayHasKey('days', $actual);
        $this->assertArrayHasKey('time', $actual);
    }

    /**
     * Assert model returns values set by mock api.
     *
     * @param array $actual
     */
    private function _assertPlanValues(array $actual)
    {
        $this->assertsame(self::PLAN_ID, $actual['id']);
        $this->assertsame($this->_isEnabled, $actual['enabled']);
        $this->assertsame(self::PLAN_DAYS, $actual['days']);
    }

    /**
     * Expect exception if api does not return a plan.
     */
    public function testIsEnabledShouldThrowExceptionIfPlanDoesNotExist()
    {
        $this->_expectException();
        $this->_SUT->isEnabled();
    }


    /**
     * isEnabled should return true.
     */
    public function testIsEnabledShouldReturnTrue()
    {
        $this->_isEnabled = true;
        $plan = $this->_getPlan();
        $this->_configureApiGetPlanByIdReturnValue($plan);

        $actual = $this->_SUT->isEnabled();
        $this->assertTrue($actual);
    }

    /**
     * isEnabled should return false.
     */
    public function testIsEnabledShouldReturnFalse()
    {
        $this->_isEnabled = false;
        $plan = $this->_getPlan();
        $this->_configureApiGetPlanByIdReturnValue($plan);

        $actual = $this->_SUT->isEnabled();
        $this->assertFalse($actual);
    }

    /**
     * Expect exception if api does not return a plan.
     */
    public function testGetScheduledDaysShouldThrowExceptionIfPlanDoesNotExist()
    {
        $this->_expectException();
        $this->_SUT->getScheduledDays();
    }

    /**
     * getScheduleDays should match api response.
     */
    public function testGetScheduledDaysShouldReturnExpectedDays()
    {
        $plan = $this->_getPlan();
        $this->_configureApiGetPlanByIdReturnValue($plan);

        $actual = $this->_SUT->getScheduledDays();
        $this->assertSame(self::PLAN_DAYS, $actual);
    }


    /**
     * Expect exception if api does not return a plan.
     */
    public function testGetScheduledTimeShouldThrowExceptionIfPlanDoesNotExist()
    {
        $this->_expectException();
        $this->_SUT->getScheduledTime();
    }

    /**
     * Expect exception if api does not return a plan.
     */
    public function testGetTypeShouldThrowExceptionIfPlanDoesNotExist()
    {
        $this->_expectException();
        $this->_SUT->getType();
    }

    /**
     * Should return type set in api mock.
     */
    public function testGetTypeShouldReturnApiResponse()
    {
        $plan = $this->_getPlan();
        $this->_configureApiGetPlanByIdReturnValue($plan);

        $actual = $this->_SUT->getType();
        $this->assertSame(self::TYPE, $actual);
    }

    /**
     * Expect exception if api does not return a plan.
     */
    public function testPrePostScriptsOkShouldThrowExceptionIfPlanDoesNotExist()
    {
        $this->_expectException();
        $this->_SUT->prePostScriptsOK();
    }

    /**
     * Expect false if config is incorrect.
     */
    public function testPrePostScriptsOkShouldReturnFalse()
    {
        $this->_setPrePostNotOk();
        $this->_configureApiGetPlanByIdReturnValue($this->_getPlan());

        $actual = $this->_SUT->prePostScriptsOK();
        $this->assertFalse($actual);
    }

    /**
     * Expect true if config is correct.
     */
    public function testPrePostScriptsOkShouldReturnTrue()
    {
        $this->_setPrePostOkConfig();
        $plan = $this->_getPlan();
        $this->_configureApiGetPlanByIdReturnValue($plan);

        $actual = $this->_SUT->prePostScriptsOK();
        $this->assertTrue($actual);
    }

    /**
     * Expect exception.
     */
    private function _expectException()
    {
        $this->expectException(Exception::class);
    }

    /**
     * configure return value for getPlanId
     *
     * @param mixed $returnValue
     */
    private function _configureApiGetPlanByIdReturnValue($returnValue)
    {

        $this->_apiMock->expects($this->any())
            ->method('getPlanById')
            ->willReturn($returnValue);
    }

    /**
     * Plan formated like api response.
     */
    private function _getPlan()
    {
        return array(
            'scheme' => array(
                'schedule' => array(
                    'daysOfWeek' => self::PLAN_DAYS,
                    'startAt' => $this->_date
                )
            ),
            'id' => self::PLAN_ID,
            'type' => self::TYPE,
            'enabled' => $this->_isEnabled,
            'options' => array(
                'backupOptions' => array(
                    'prePostCommands' => $this->_prePostCommands
                )
            )
        );
    }

    /**
     * Set pre post scripts with expected config.
     */
    private function _setPrePostOkConfig()
    {
        $expectedConfig = array(
            'workingDirectory' => '',
            'continueOnCommandError' => false,
            'commandArguments' => '{RESOURCE_ID}',
            'waitCommandComplete' => true,
            'command' => '/usr/lib/Acronis/BackupAndRecovery/webcpprecapture'
        );

        $this->_prePostCommands = array(
            'usePreCommands' => true,
            'usePostCommands' => true,
            'preCommands' => $expectedConfig,
            'postCommands' => $expectedConfig
        );
    }

    /**
     * Set prepost scripts not ok.
     */
    private function _setPrePostNotOk()
    {
        $falseConfig = array(
            'workingDirectory' => '',
            'continueOnCommandError' => false,
            'commandArguments' => '',
            'waitCommandComplete' => '',
            'command' => ''
        );

        $this->_prePostCommands = array(
            'preCommands' => $falseConfig,
            'postCommands' => $falseConfig,
            'usePreCommands' => false,
            'usePostCommands' => false,
        );
    }
}
