<?php

/**
 * InterWorx Hosting Control Panel
 *
 * <pre>
 * +----------------------------------------------------------------------+
 * | Copyright (c) 2000-2010 InterWorx L.L.C., All Rights Reserved.       |
 * +----------------------------------------------------------------------+
 * | Redistribution and use in source form, with or without modification  |
 * | is NOT permitted without consent from the copyright holder.          |
 * |                                                                      |
 * | THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS "AS IS" AND |
 * | ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,    |
 * | THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A          |
 * | PARTICULAR PURPOSE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,    |
 * | EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,  |
 * | PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR   |
 * | PROFITS; OF BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY  |
 * | OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT         |
 * | (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE    |
 * | USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH     |
 * | DAMAGE.                                                              |
 * +----------------------------------------------------------------------+
 * </pre>
 *
 * @package InterWorx
 * @author  Malin Lorch <mlorch@liquidweb.com>
 * @date    Aug 18, 2020
 */

require_once(__DIR__ . '/Acronis.php');

/**
 *
 *
 * @package Interworx
 * @subpackage Plugin_Acronis
 */
class RecoveryFilesTest extends AcronisTest
{
    const CREATE_ALT_LOCATION_FUNCTION = '_createRestoreAltLocationDir';
    const DELETE_NEWER_FUNCTION = '_deleteNewerFiles';
    const API_RECOVER_DIRECTORIES = 'recoverDirectories';
    const RESTORE_DUMPS = '_restoreDumps';
    const CACHE_GETBACKUPS = 'getBackupById';

    const UNIX_USER = 'unixxuser';
    const BACKUP_ID = 'somebackupid';
    const DB_NAME = 'somedatabasename';

    const DUMPS_DIR_ID = 'dumpsdirid';
    const SERVER_DIR_ID = 'serverdirid';
    const ANOTHER_DIR_ID = 'anotherdirid';

    private $_folderContentDumps = array(
        array(
            'displayName' => 'server-dumps',
            'id' => self::SERVER_DIR_ID
        ),
        array(
            'displayName' => 'addedbyt-dumps',
            'id' => self::ANOTHER_DIR_ID
        )
    );

    private $_folderContentServerDumps = array(
        array(
            'displayName' => 'mysql'
        ),
        array(
            'displayName' => 'another'
        )
    );

    private $_folderContentAnotherDumps = array(
        array(
            'displayName' => 'somedatabase'
        ),
        array(
            'displayName' => 'anotherdatabase'
        )
    );


    private $_SUT;
    private $_ApiMock;

    /**
     * set up before test.
     */
    public function setUp()
    {
        $this->_ApiMock = $this->_createApiMock();
        $this->_SUT = $this->_getSutPartialMock($this->_ApiMock);

        $this->_SUT->expects($this->any())
            ->method('getDbDumpsId')
            ->willReturn(self::DUMPS_DIR_ID);
    }

    /**
     * Test getDbs should exclude mysql
     */
    public function testGetDatabasesShouldExcludeMysql()
    {
        $actual = $this->_SUT->getDbs();
        $this->assertCount(2, $actual);

        foreach ($actual as $dbsInfo) {
            $actualDbs = $dbsInfo['dbNames'];
            $this->assertNotContains('mysql', $actualDbs);
        }
    }

    /**
     * Test getSwDbs should return right info
     */
    public function testGetSwDbsShouldOnlyReturnSwUser()
    {

        $unixUser = 'addedbyt';

        $result = $this->_SUT->getSwDbs($unixUser);
        $actualUser = $result['unixUser'];
        $actualDbs = $result['dbNames'];

        $this->assertSame($unixUser, $actualUser);
        $this->assertContains('somedatabase', $actualDbs);
        $this->assertContains('anotherdatabase', $actualDbs);
    }

    /**
     * Test getSwDbs should throw exception if sw user does not exist
     */
    public function testGetSwDbsShouldThrowExceptionIfSwUserDoesNotExist()
    {
        $this->expectException(Exception::class);

        $unixUser = 'nonexistinguser';
        $this->_SUT->getSwDbs($unixUser);
    }

    /**
     * get system under test.
     *
     * @param mixed $ApiMock
     */
    protected function _getSutPartialMock($ApiMock)
    {
        $SUT = $this->_getSut();

        $CacheMock = $this->_createCacheMock();

        $CacheMock->expects($this->any())
            ->method(self::CACHE_GETBACKUPS)
            ->with($this->equalTo(self::BACKUP_ID))
            ->willReturn(array('somethings'));

        $SUT->expects($this->any())
            ->method(self::ACTION_GET_CACHE)
            ->willReturn($CacheMock);

        $SUT->expects($this->any())
            ->method(self::ACTION_GET_API)
            ->willReturn($ApiMock);

        return $SUT;
    }


    /**
     * Set SUT with constructor args.
     */
    private function _getSut()
    {
        $SUT = $this->getMockBuilder(Model_RecoveryPoint_Databases::class)
            ->setMethods([
                '__construct',
                self::ACTION_GET_API,
                self::ACTION_GET_CACHE,
                self::DELETE_NEWER_FUNCTION,
                self::CREATE_ALT_LOCATION_FUNCTION,
                self::RESTORE_DUMPS,
                'getDbDumpsId'
            ])
            ->setConstructorArgs([self::BACKUP_ID])
            ->getMock();

        return $SUT;
    }

    /**
     * create api mock.
     *
     * @return Mock
     */
    private function _createApiMock()
    {
        $Mock = $this->createMock(Model_RecoveryPoint_Api::class);

        // first param = expected argument,
        // second param = return value.
        $valueMap = array(
            [self::DUMPS_DIR_ID, $this->_folderContentDumps],
            [self::ANOTHER_DIR_ID, $this->_folderContentAnotherDumps],
            [self::SERVER_DIR_ID, $this->_folderContentServerDumps]
        );

        $Mock->expects($this->any())
            ->method('getDirectoryContent')
            ->will(
                $this->returnValueMap($valueMap)
            );

        return $Mock;
    }
}
