<?php

/**
 * InterWorx Hosting Control Panel
 *
 * <pre>
 * +----------------------------------------------------------------------+
 * | Copyright (c) 2000-2010 InterWorx L.L.C., All Rights Reserved.       |
 * +----------------------------------------------------------------------+
 * | Redistribution and use in source form, with or without modification  |
 * | is NOT permitted without consent from the copyright holder.          |
 * |                                                                      |
 * | THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS "AS IS" AND |
 * | ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,    |
 * | THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A          |
 * | PARTICULAR PURPOSE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,    |
 * | EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,  |
 * | PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR   |
 * | PROFITS; OF BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY  |
 * | OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT         |
 * | (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE    |
 * | USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH     |
 * | DAMAGE.                                                              |
 * +----------------------------------------------------------------------+
 * </pre>
 *
 * @package InterWorx
 * @author  Malin Lorch <mlorch@liquidweb.com>
 * @date    May 12, 2020
 */

require_once(__DIR__ . '/Acronis.php');

/**
 *
 *
 * @package Interworx
 * @subpackage Plugin_Acronis
 */
class RecoveryFilesTest extends AcronisTest
{
    const CREATE_ALT_LOCATION_FUNCTION = '_createRestoreAltLocationDir';
    const DELETE_NEWER_FUNCTION = '_deleteNewerFiles';
    const API_RECOVER_DIRECTORIES = 'recoverDirectories';
    const RESTORE_DUMPS = '_restoreDumps';
    const CACHE_GETBACKUPS = 'getBackupById';

    const UNIX_USER = 'unixxuser';
    const BACKUP_ID = 'somebackupid';
    const DB_NAME = 'somedatabasename';

    private $_SUT;
    private $_ApiMock;

    /**
     * set up before test.
     */
    public function setUp()
    {
        $this->_ApiMock = $this->_createApiMock();
        $this->_SUT = $this->_getSutPartialMock($this->_ApiMock);
    }

    /**
     * Test getDirectoryContent. SHould throw exception if cache does not contain backup.
     */
    public function testGetDirectoryContentShouldReturnArray()
    {
        $dirId = 'dirid';
        $actual = $this->_SUT->getDirectoryContent($dirId);
        $this->assertInternalType('array', $actual);
    }

    /**
     * Test getDirectoryContent. SHould throw exception if cache does not contain backup.
     */
    public function testGetRootContentShouldReturnArray()
    {
        $actual = $this->_SUT->getRootContent();
        $this->assertInternalType('array', $actual);
    }


    /**
     * test recoverDirectories should call api method.
     */
    public function testRecoverDirectories()
    {
        $this->_SUT->expects($this->never())
            ->method(self::DELETE_NEWER_FUNCTION);

        $this->_ApiMock->expects($this->once())
            ->method(self::API_RECOVER_DIRECTORIES);

        $this->_SUT->recoverDirectories(array('dir'));
    }

    /**
     * test recoverDirectories should call api method.
     */
    public function testRecoverDirectoriesAndDelete()
    {
        $this->_SUT->expects($this->once())
            ->method(self::DELETE_NEWER_FUNCTION);

        $this->_ApiMock->expects($this->once())
            ->method(self::API_RECOVER_DIRECTORIES);

        $deleteNewer = true;
        $this->_SUT->recoverDirectories(array('dir'), $deleteNewer);
    }

    /**
     * Restore to alt location, should create directory & call api.
     */
    public function testRestoreToAltLocation()
    {
        $expectedPath = '/home/acronis_temp';
        $this->_SUT->expects($this->once())
            ->method(self::CREATE_ALT_LOCATION_FUNCTION)
            ->with($this->stringContains($expectedPath));

        $this->_ApiMock->expects($this->once())
            ->method('recoverDirectoriesToAltLocation');

        $this->_SUT->restoreToAltLocation(array('dir'));
    }

    /**
     * test getRestoreAltLocationPa
     */
    public function testGetRestoreAltLocationPath()
    {
        $actual = $this->_SUT::getRestoreAltLocationPath(IW::SW());
        $expected = '/home/acronis_temp';
        $this->assertSame($expected, $actual);
    }


    /**
     * get system under test.
     *
     * @param mixed $ApiMock
     */
    protected function _getSutPartialMock($ApiMock)
    {
        $SUT = $this->_getSut();

        $CacheMock = $this->_createCacheMock();

        $CacheMock->expects($this->any())
        ->method(self::CACHE_GETBACKUPS)
        ->with($this->equalTo(self::BACKUP_ID))
        ->willReturn(array('somethings'));

        $SUT->expects($this->any())
            ->method(self::ACTION_GET_CACHE)
            ->willReturn($CacheMock);

        $SUT->expects($this->any())
            ->method(self::ACTION_GET_API)
            ->willReturn($ApiMock);

        return $SUT;
    }


    /**
     * Set SUT with constructor args.
     */
    private function _getSut()
    {
        $SUT = $this->getMockBuilder(Model_RecoveryPoint_Files_Nodeworx::class)
            ->setMethods([
                '__construct',
                self::ACTION_GET_API,
                self::ACTION_GET_CACHE,
                self::DELETE_NEWER_FUNCTION,
                self::CREATE_ALT_LOCATION_FUNCTION,
                self::RESTORE_DUMPS
            ])
            ->setConstructorArgs([self::BACKUP_ID])
            ->getMock();

        return $SUT;
    }

    /**
     * create api mock.
     */
    private function _createApiMock()
    {
        $Mock = $this->createMock(Model_RecoveryPoint_Api::class);

        $Mock->expects($this->any())
        ->method('getDirectoryContent')
        ->willReturn(array());

        return $Mock;
    }
}
