<?php

/**
 * InterWorx Hosting Control Panel
 *
 * <pre>
 * +----------------------------------------------------------------------+
 * | Copyright (c) 2000-2010 InterWorx L.L.C., All Rights Reserved.       |
 * +----------------------------------------------------------------------+
 * | Redistribution and use in source form, with or without modification  |
 * | is NOT permitted without consent from the copyright holder.          |
 * |                                                                      |
 * | THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS "AS IS" AND |
 * | ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,    |
 * | THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A          |
 * | PARTICULAR PURPOSE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,    |
 * | EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,  |
 * | PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR   |
 * | PROFITS; OF BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY  |
 * | OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT         |
 * | (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE    |
 * | USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH     |
 * | DAMAGE.                                                              |
 * +----------------------------------------------------------------------+
 * </pre>
 *
 * @package InterWorx
 * @author  Malin Lorch <mlorch@liquidweb.com>
 * @date    Apr 28, 2020
 */

require_once(__DIR__ . '/Acronis.php');
require_once(__DIR__ . '/../../Model/Server/Server.php');
require_once(__DIR__ . '/../../Model/Server/Api.php');

/**
 *
 *
 * @package Interworx
 * @subpackage Plugin_Acronis
 */
class ServerTest extends AcronisTest
{
    const ACTION_ADD_RESOURCE = '_addResourceToPlan';
    const ACTION_PREPOST_OK = '_prePostScriptsOk';
    const RESOURCE_ID = 'resourceifd';
    const INSTANCE_ID = 'instanceId';
    const CACHE_PLAN_ID = 'cachedplanid';
    const API_PLAN_ID = 'apiplanid';
    const NEW_PLAN_ID = 'newplanid';

    const LAST_BACKUP = 0;
    const NEXT_BACKUP = 0;
    const ONE_GB_IN_BYTES = 1073741824;
    const IS_ONLINE = true;

    private $_SUT;
    private $_Cache;
    private $_Api;

    /**
     * Set up before each test.
     */
    public function setUp()
    {
        $this->_SUT = $this->_getSutPartialMock();
    }



    /**
     * Test getPlanId - should return cached value if set.
     */
    public function testGetPlanIdShouldReturnCachedId()
    {
        $expected = self::CACHE_PLAN_ID;
        $this->_configCacheGetPlanId($expected);

        $actual = $this->_SUT->getPlanId();
        $this->assertSame($expected, $actual);
    }


    /**
     * Configure cache getPlanId return value.
     *
     * @param string $value
     */
    private function _configCacheGetPlanId($value)
    {
        $this->_Cache->expects($this->any())
            ->method('getPlanId')
            ->willReturn($value);
    }

    /**
     * Test getPlanId - should return api planid if no cache.
     */
    public function testGetPlanIdShouldGetAssignedPlanIdFromApi()
    {
        $expected = self::API_PLAN_ID;
        $this->_configureApiGetBackupPlansReturnValue($expected);

        $actual = $this->_SUT->getPlanId();
        $this->assertSame($expected, $actual);
    }

    /**
     * configure api return value.
     *
     * @param string $planId
     */
    private function _configureApiGetBackupPlansReturnValue($planId = null)
    {
        $plans = array();

        if (isset($planId)) {
            $plans[] = array(
                'planId' => $planId
            );
        }

        $this->_Api->expects($this->once())
            ->method('getBackupPlans')
            ->willReturn($plans);
    }

    /**
     * Get plan id should add resource to plan if resource isn't already assigned to a plan.
     *
     */
    public function testGetPlanIdShouldAssignResourceToPlan()
    {

        $this->_configureApiGetBackupPlansReturnValue(null);

        $expected = self::NEW_PLAN_ID;
        $this->_configureGetApplicablePlansReturnValue($expected);

        $this->_SUT->expects($this->once())
            ->method(self::ACTION_ADD_RESOURCE);

        $this->_SUT->getPlanId();
    }

    /**
     * Configure api return value.
     *
     * @param string $planId
     */
    private function _configureGetApplicablePlansReturnValue($planId = null)
    {
        $plans = array();
        if (isset($planId)) {
            $plans[] = array('ID' => $planId);
        }

        $returnValue = array(
            'Plans' => $plans
        );

        $this->_Api->expects($this->once())
            ->method('getApplicablePlans')
            ->willReturn($returnValue);
    }

    /**
     * getPlanId should throw exception if no plans exists.
     */
    public function testGetPlanIdShouldThrowExceptionIfNoPlanExists()
    {
        $this->_configureApiGetBackupPlansReturnValue(null);
        $this->_configureGetApplicablePlansReturnValue(null);

        $this->expectException(Exception::class);
        $this->_SUT->getPlanId();
    }

    /**
     * Test getBackupStatus. Should return array with specific keys.
     * Not comparing value becuase of date formatter used in model.
     */
    public function testGetBackupStatus()
    {
        $this->_expectApiCallGetBackupStatus();
        $actual = $this->_SUT->getBackupStatus();
        $this->assertArrayHasKey('lastStatus', $actual);
        $this->assertArrayHasKey('lastBackup', $actual);
        $this->assertArrayHasKey('nextBackup', $actual);
    }


    /**
     * expect api call to getBackupStatus once.
     */
    private function _expectApiCallGetBackupStatus()
    {
        $this->_Api->expects($this->once())
            ->method('getStatus')
            ->with(self::RESOURCE_ID)
            ->willReturn(
                array(
                    array(
                        'lastStatus' => 'lastStatus',
                        'lastBackup' => 'lastBackup',
                        'nextBackup' => 'nextBackup'
                    )
                )
            );
    }

    /**
     * backupNotScheduled - should return true when api res excludes nextBackup
     */
    public function testBackupNotScheduledShouldReturnTrueWhenApiResExcludesNextBackup()
    {
        $shouldIncludeNextBackup = false;
        $this->_formatApiResNextBackup($shouldIncludeNextBackup);

        $actual = $this->_SUT->backupNotScheduled();
        $this->assertTrue($actual);
    }

    /**
     * backupNotScheduled - should return false when api res includes nextBackup
     */
    public function testBackupNotScheduledShouldReturnFalseWhenApiResIncludesNextBackup()
    {
        $shouldIncludeNextBackup = true;
        $this->_formatApiResNextBackup($shouldIncludeNextBackup);

        $actual = $this->_SUT->backupNotScheduled();
        $this->assertFalse($actual);
    }

    /**
     * expect api call to getBackupStatus once.
     *
     * @param boolean $shouldIncludeNextBackup
     */
    private function _formatApiResNextBackup($shouldIncludeNextBackup)
    {
        $date = date(DATE_RFC2822);
        $apiRes = array(
            'lastStatus' => 'ok',
            'lastStartTime' => $date
        );

        if ($shouldIncludeNextBackup) {
            $apiRes['nextStartTime'] = $date;
        }

        $this->_Api->expects($this->once())
            ->method('getStatus')
            ->with(self::RESOURCE_ID)
            ->willReturn(
                array($apiRes)
            );
    }

    /**
     *
     */
    public function testGetStorageData()
    {
        $result = $this->_SUT->getStorageData();
        $this->assertArrayHasKey('size', $result);

        $actual = $result['size'];
        $expected = '1 GB';
        $this->assertSame($expected, $actual);
    }


    /**
     * isProtected should return true.
     */
    public function testIsProtectedShouldReturnTrue()
    {
        $actual = $this->_SUT->isProtected();
        $this->assertTrue($actual);
    }


    /**
     * test is online
     */
    public function testIsOnline()
    {
        $actual = $this->_SUT->isOnline();
        $expected = self::IS_ONLINE;

        $this->assertSame($expected, $actual);
    }

    /**
     * Test get next backup
     */
    public function testGetNextBackup()
    {
        $actual = $this->_SUT->getNextBackup();
        $expected  = self::NEXT_BACKUP;

        $this->assertSame($expected, $actual);
    }

    /**
     * test get last backup
     */
    public function testGetLastBackup()
    {
        $actual = $this->_SUT->getLastBackup();
        $expected = self::LAST_BACKUP;

        $this->assertSame($expected, $actual);
    }

    /**
     * test Get instance id
     */
    public function testGetInstanceId()
    {
        $actual = $this->_SUT->getInstanceId();
        $expected = self::INSTANCE_ID;

        $this->assertSame($expected, $actual);
    }

    /**
     * test get resource id
     */
    public function testGetResourceId()
    {
        $actual = $this->_SUT->getResourceId();
        $expected = self::RESOURCE_ID;

        $this->assertSame($expected, $actual);
    }

    /**
     * Get sut as partial mock - make sure getApi & getCache etc returns mocks.
     *
     * @param boolean $scriptsOk
     * @return MockObject
     */
    protected function _getSutPartialMock($scriptsOk = true)
    {
        $this->_Cache = $this->_createCacheMock();
        $this->_Api = $this->_getMockApi();
        $this->_setApiStatus();

        $methodsToMock = array(
            '__construct',
            self::ACTION_GET_API,
            self::ACTION_GET_CACHE,
            self::ACTION_GET_PLUGIN,
            self::ACTION_PREPOST_OK,
            self::ACTION_ADD_RESOURCE
        );

        $SUT = $this->getMockBuilder(Model_Server_Server::class)
            ->setMethods($methodsToMock)
            ->setConstructorArgs([self::INSTANCE_ID, self::RESOURCE_ID])
            ->getMock();

        $SUT->expects($this->any())
            ->method(self::ACTION_GET_CACHE)
            ->willReturn($this->_Cache);

        $SUT->expects($this->any())
            ->method(self::ACTION_GET_API)
            ->willReturn($this->_Api);

        $SUT->expects($this->any())
            ->method(self::ACTION_PREPOST_OK)
            ->willReturn($scriptsOk);

        return $SUT;
    }

    /**
     * get mock api.
     */
    private function _getMockApi()
    {
        $Mock = $this->createMock(Model_Server_Api::class);

        $Mock->expects($this->any())
            ->method('getStorageData')
            ->willReturn(array(
                'occupied_data_size' => self::ONE_GB_IN_BYTES
            ));

        return $Mock;
    }

    private function _setApiStatus($status = null)
    {
        $this->_Api->expects($this->any())
            ->method('getDevice')
            ->willReturn($this->_getDeviceInfo($status));
    }

    /**
     * Get device info.
     */
    private function _getDeviceInfo($status = null)
    {
        return array(
            'id' => self::RESOURCE_ID,
            'lastBackup' => self::LAST_BACKUP,
            'nextBackup' => self::NEXT_BACKUP,
            'online' => self::IS_ONLINE,
            'instanceId' => self::INSTANCE_ID,
            'status' => isset($status) ? $status : 'ok'
        );
    }
}
